function bel_num_out = make_joint_from_condit(bel_num_in)

% enter ONE belief number 
% verify if it is a conditional belief object
% if no, do nothing and bel_num_out = 0
% if yes, every conditional is ballooned on joint space
% and then conjunctively combined and bel_num_out is number new belief
% not done that way as too slow.
% I use the GBT/DRC relations

% NCI

%
%   Version : Feb 10,03.
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global frame structure mapping attribute variable belief
global nbr_bit_per_word

ind_bel = pin_manipulation('belief','num2ind',bel_num_in);

if isempty(belief(ind_bel).conditioning_variable)
	bel_num_out = belief(ind_bel).number;
	return
end

nbr_bp = length(belief(ind_bel).bp);

% being sure we have the m for each potential. Done in run_VBS
% create_m_if_absent(ind_bel)

% localizing the bbm functions
for i = 1:nbr_bp
	ind_bbm(i) = strmatch('m',{belief(ind_bel).bp(i).form(:).type},'exact');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% conjunctive combi using the underlying algo of the GBT/DRC, not brute force.

% suppose: card = 3 and 3 bp's (m1,m2,m3)

% bbm    m1(1) m1(2)    m2(1) m2(2) m2(3)    m3(1) m3(2)
% foset    4     5        2     3     1        6     7 

%   new_bbm is :            allocated to nbr	bits     	 	new bits
%   m1(1) * m2(1) * m3(1)      	6	2  	4  		110 010 100
%   m1(2) * m2(1) * m3(1)		6	2	5  		110 010 101
%   m1(1) * m2(2) * m3(1)		6 	3	4  		110 011 100
%   m1(2) * m2(2) * m3(1)		6 	3 	5  		110 011 101
%   m1(1) * m2(3) * m3(1)		6	1	4  		110 001 100		100001110
%   m1(2) * m2(3) * m3(1)		6	1	5  		110 001 101

%   m1(1) * m2(1) * m3(2)		7	2  	4  		111 010 100  
%   m1(2) * m2(1) * m3(2)		7	2	5  		111 010 101
%   m1(1) * m2(2) * m3(2)		7	3	4  		111 011 100
%   m1(2) * m2(2) * m3(2)		7	3	5  		111 011 101
%   m1(1) * m2(3) * m3(2)		7	1	4  		111 001 100
%   m1(2) * m2(3) * m3(2)		7	1	5  		111 001 101

% worlds ordering
% T=1,X=1-3, T=2,X=1-3,  T=3,X=1-3
% 	w	T	X
%	1	1	1
%	2	1	2
%	3	1	3
%	4	2	1
%	5	2	2
%	6	2	3
%	7	3	1
%	8	3	2
%	9	3	3
%
% identical to
%  		T=1	T=2	T=3
% X=1	1	4	7
% X=2	2	5	8
% X=3	3	6	9
%
% obtained by defining    .domain_variable = [X T]    in
% belief().domain_variable = [belief().domain_variable    belief().conditioning_variable];
% leftmost var in domain_variable is the fastest changing var

% the mass m1(1) * m2(3) * m3(1)		6	1	4  	their bit vevtors are	0 1 1, 1 0 0, 0 0 1
% is allocated to wolrds 2 3     4     9, thus bit_vector    0 1 1   1 0 0   0 0 1
%
% fast algorithm, only for final card <= nbr_bit_per_word
% I store [6 2 4 ; 6 2 5 ; 6 3 4  ...]  
% for 6 2 4 , the bit_vector is  0 1 1  0 1 0   0 0 1, thus 2 + 4 + 16 + 256 = 278
% I build 4*64 + 2*8 + 6*1 = 256 + 16 + 6 = 278, the long_number of new foset
% if final card <= 50 easy
% else I must go thru the bits probably

card = prod(belief(ind_bel).domain_cardinality);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% building the new bbm
new_bbm = belief(ind_bel).bp(1).form(ind_bbm(1)).values;
for i = 2:nbr_bp
	new_bbm = kron(belief(ind_bel).bp(i).form(ind_bbm(i)).values,new_bbm);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% building the fosets
if card*nbr_bp <= nbr_bit_per_word
	% the fast algo, but size essential
	new_foc = belief(ind_bel).bp(1).form(ind_bbm(1)).focal_sets;  % [4 ; 5]
	for i = 2:nbr_bp	
		new_foc = [ kron(belief(ind_bel).bp(i).form(ind_bbm(i)).focal_sets,   ones(size(new_foc,1),1))    ...
		kron(ones(size(belief(ind_bel).bp(i).form(ind_bbm(i)).focal_sets,1),1),new_foc) ];
		% i = 2:   [ 2 4 ; 2 5 ; 3 4 ; 3 5 ; 1 4 ; 1 5 ] .... 
		% i = 3:   [ 6 2 4 ; 6 2 5 ; 6 3 4 ; 6 3 5 ; 6 1 4 ; 6 1 5 ; 7 2 4 ; 7 2 5 ; 7 3 4 ; 7 3 5 ; 7 1 4 ; 7 1 5 ]
	end
	for i = 1:size(new_foc,1)
		fosets(i,1) = sum(new_foc(i,:).*pow2(card).^[0:nbr_bp-1]);		
	end
else
	% the algo thru the bits, VERY slow	
	new_foc = belief(ind_bel).bp(1).form(ind_bbm(1)).focal_sets;
	bit_foc = long_num2bit(new_foc,card);	
	for i = 2:nbr_bp	
		bit_foc_now  = long_num2bit(belief(ind_bel).bp(i).form(ind_bbm(i)).focal_sets,card);
		ik = 0;
		bit_foc_new = [];
		for j = 1:size(bit_foc_now,1)
			for k = 1:size(bit_foc,1)
				ik = ik + 1;
				bit_foc_new(ik,:) = [bit_foc_now(j,:) bit_foc(k,:)];
			end
		end
		bit_foc = bit_foc_new;
	end
	fosets = long_bit2num(bit_foc,card*nbr_bp);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% storing the new belief
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

bel_ind_out = make_empty('belief');
useless = make_empty('bp',bel_ind_out);
useless = pin_manipulation('belief','all',[]);

bel_num_out = belief(bel_ind_out).number;	

% the order of the var are domain_var then conditionioning_var
belief(bel_ind_out).domain_variable = [belief(ind_bel).domain_variable    belief(ind_bel).conditioning_variable];

ind_var = pin_manipulation('variable','num2ind',belief(bel_ind_out).domain_variable);
num_fr = pin_manipulation('frame','num2ind',[variable(ind_var).frame]);
belief(bel_ind_out).domain_cardinality = get_cardinal_var(num_fr);
get_its_dci(bel_ind_out);

belief(bel_ind_out).bp(1).form(1).type = 'm';
belief(bel_ind_out).bp(1).form(1).values = new_bbm;
belief(bel_ind_out).bp(1).form(1).focal_sets = fosets;


