function [m_final,foc_final,list_number_var,list_fr_card] = compute_unmixed_combi(...
index_belief, bp_index, form_index, which)
% compute pure conjucitve (which = 'and') 
% or pure disjunctive combination (which = 'or') 

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%
global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global nbr_bit_per_word
global length_one_focal % space needed for one B(:,:,jj)

% finding the variables of the bp's without repetition
list_number_var 	= unique([belief(index_belief).domain_variable]);

list_index_var 		= pin_manipulation('variable','num2ind',list_number_var);
list_number_frame 	= [variable(list_index_var).frame];
list_index_frame 	= pin_manipulation('frame','num2ind',list_number_frame);
list_fr_card 		= [frame(list_index_frame).cardinality];

list_bbf_unique(:,2:4) = [index_belief bp_index form_index];

nbr_worlds = prod(list_fr_card);
% nbr_worlds = nbr of worlds in the common frame

[nbr_full_words, remain] =  words_for_set(nbr_worlds);
% bit representation parameters,nbr_full_words of nbr_bit_per_word bits + remain bits in next one

nbr_words = nbr_full_words + 1;
% nbr of words needed to store a long_number (a set)

universe = get_universe(nbr_worlds);
% the representation of the overal frame, used to put 0 on the unused bits

nbr_m = length(index_belief);


B = [];
lengthB = 1;
m = 1;

for iv = 1:nbr_m  % for every involved bp
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% computing the masses by products
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	bel_ind = list_bbf_unique(iv,2);
	bp_ind = list_bbf_unique(iv,3);
	form_ind = list_bbf_unique(iv,4);
	m1 = belief(bel_ind).bp(bp_ind).form(form_ind).values;
	m = kron(m1,m);
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% vacuous extension of every belief function on common frame
	% building the new focal sets on the common frame
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% handling the slected/ordered cases for the focal sets
	if strcmp(belief(list_bbf_unique(iv,2)).bp(list_bbf_unique(iv,3)).form(list_bbf_unique(iv,4)).representation,...
		'selected')
		foc_in = belief(list_bbf_unique(iv,2)).bp(list_bbf_unique(iv,3)).form(list_bbf_unique(iv,4)).focal_sets;
	else % bbm given on ordered sets. I build the focal sets
		cardinal = prod([belief(list_bbf_unique(iv,2)).domain_cardinality]);
		foc_in = [0:pow2(cardinal)-1]';
		% BEWARE: when ordered, cardinal always lower than nbr_bit_per_word, so ONE word. 
	end

	% used_var 	- the variable index of unextended bp, same order as belief().domain_variable
	% new_var	- the variable index of the new variables, whatever order
	% result on variable with order as in list_index_var 
	% {list_index_var} = union(used_var, new_var) same sets
	
	list_new = setdiff(list_index_var,belief(list_bbf_unique(iv,2)).domain_variable);
	
	% the focal sets extended
	focal_prod_sp(iv).foc_out =  vacuous_ext_long_focal  (foc_in, ...
	belief(list_bbf_unique(iv,2)).domain_variable, list_new, list_index_var, list_fr_card);
	
	% focal_prod_sp(iv).foc_out(ii,:) = list of the ii focal sets for bp(iv)
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% building the matrix B with all the focal sets
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	
	C = []; 
	D = [];
	for ii = 1:length(m1)
		C = [C ; B];
		D = [D ;  ones(lengthB,1)*focal_prod_sp(iv).foc_out(ii,:) ] ; 
	end
	B = [C D]; % B is a nbrfoc x (nbr_words*nbr_m)
	[lengthB useless] = size(B);
	
end

[lengthB nc] = size(B);
B = reshape(B,lengthB, nc/nbr_m,nbr_m); % B is a nbrfoc x nbr_words x nbr_m

% B : B(r,s,t) with s one of the numbers that store one set (bits representation)
% r is 1 to length(m), 	corresponds to the m(r)
% t is 1 to nbr_m, 		corresponds to the nbr_m  bp's involved

foc_new = bitand(B());

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% adding masses given to same focal sets
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

% mass m(i) given to foc_new(i,:)
% there are duplicate in foc_new(i,:)

sorted_data = sortrows([foc_new m]);
[nr nc] = size(foc_new);

mm = sorted_data(:,nc+1);

indx = [0 find(sum(abs(diff(sorted_data(:,1:nc))),2))' nr];
% these are the position where focal sets change
% with sorted_data = 
%      1     3
%      1     3
%      1     3
%      2     1
%      2     4
%      2     4
% we get indx = [0 3 4 6], so e must add mm(1:3) mm(4:4) mm(5:6)

for i = 1:length(indx)-1
	m_final(i) = sum(mm(indx(i)+1:indx(i+1)));
	foc_final(i,1:nc) = sorted_data(indx(i)+1,1:nc);
end

