function run_distinct_combi(which_combi)
% root for the conjunctive and disjunctive combination of bp's and their negation 
% where bp's are produced by distinct pieces of evidence.
% which_combi = 	'mixed' 	we use AND, OR, NOT
% 					'and' 		we use only AND
% 					'or' 		we use only OR


% Combinations perfomred on product space, so space greedy.
% Maximum 99 different bp's.

% Not optimal if there are only and (or only or) operations.
% For conjunction only, use run_distinct_conj
% for disjunction only, use run_distinct_disj

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global length_one_focal
global gui_position  figure_data  flags  help_level help_label

% I build the logical expression applicable to the involved bp's
% so a list of belief, of bp in belief, and of forms in bp is displayed
% then the expr is bult, and the involved variables are determined.
% In case the expr is not ok (var_ok = 0), program states it, and cycle
% end building the expr whith the end button
% quit the calculator by putting q in the query box.

% Results are stored in a new belief object

% if you want to marginalize it, proceed with run_marginalization
% which creates a new belief object
% and if you want to see results use the output options

length_one_focal = 9;
% nbr of symbols to represent B(:,:,jj), here 9

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% get the figure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

calculator_handle = figure('name','Combining Beliefs from Distinct Sources');
set(calculator_handle,'position',gui_position(:,5));
ind_fig = length(figure_data) + 1;
figure_data(ind_fig).name = 'Combining Beliefs from Distinct Sources';
figure_data(ind_fig).handle = calculator_handle;
figure_data(ind_fig).position = gui_position(:,5);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% put its menus
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

k=0;
k=k+1;make_fig_menu(ind_fig,k,'File',0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Quit',1,'general_clb(1)','on');
% 
% k=k+1;make_fig_menu(ind_fig,k, help_label,0,'','on');
% k=k+1;make_fig_menu(ind_fig,k,'Window Helps On', 3,'general_clb(2)','on');
% k=k+1;make_fig_menu(ind_fig,k,'Window Helps Off',3,'general_clb(3)','off');

menu_manager('create',ind_fig)

hd_help = forced_display_helps(0,25);
% there is only one help so I just show it 
% display_helps put the Root Window in front with a help text

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% list of all belief (and all bp)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ok = check_avaiable({'belief','variable','frame'});

if ok 
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% the logical expression and mass request
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
	
	go_on = 1;
	while go_on
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% what to do with resulting belief
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		clear queries default_answer
		
		title = ['Distinct combination : ',which_combi,' form.'];
		queries{1} ='Short name for resulting belief';
		queries{2} ='Full name for resulting belief (default = short name)';
		
		pin = get_next_pin('belief');
		bel_nbr = pin(3);
		
		default_answer{1} = strcat('B',num2str(bel_nbr));
		default_answer{2} = default_answer{1};
		
		[answer,choice17] = dialog_queries(calculator_handle, title, queries, ...
		default_answer,[0 0],[],2,[],[],[],[]);
		
		if choice17 ~=1 % cancel at first query display
			go_on = 0; % we quit calculator, back to TBM
			delete(calculator_handle);
			figure_data(ind_fig) = [];
			delete(hd_help)
		else
	
			% entering the beliefs to be combined
			get_bel_bp_forms
			
			if proceed % get_bel_bp_forms generate proceed == 0 when not OK or cancel
				
				% produced by get_bel_bp_forms: [index_belief' bp_index' form_index' num_belief']
				belief_list = {belief(index_belief).full_name}; 			
				
				switch which_combi
				case {'and','or'}		
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					%
					% the pure AND or OR combinations
					%
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%				 
					part_andor_dist_combi
					
				case 'mixed'
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					%
					% the mixed combinations
					%
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					part_mixed_dist_combi
					
					% XXXXX to be verify that expr does not end with a ; as it is not ok for eval
					
				end % switch which_combi	
			end
		
			if choice19 == 1 % from get_bel_bp_forms or part_mixed_dist_combi, I did not cancel combination request
				if expr_ok == 1
					
					log_expr_wrong = 0; % so we don't repeat the reading of expr
					
					ibel = make_empty('belief');			
					useless = pin_manipulation('belief','all',[]);
					ipot = make_empty('potential',ibel);	
					bel_nbr = belief(ibel).number;
					
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					%
					% compute m: THE REAL COMPUTATION
					%
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					
					fprintf(1,'\n',[]);
					fprintf(1,'Distinct Combination: formula\n',[]);
					fprintf(1,'%s \n',eval_expr);
					
					tic % time for the combi
					[m_final,foc_final,var_num_final,list_fr_card ] = ...
					compute_distinct_combi(eval_expr,list_bbf_unique,list_bbf_labels);
					disp('The time used for the Distinct Combination.')				
					toc
					
					% 			fprintf(1,'The bba:\n',[]);
					% 			fprintf(1,'%g  ',m_final);
					% 			fprintf(1,'\n',[]);
					% 			display_set(foc_final,prod(list_fr_card))
					
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					%
					% saving results
					%
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					
					% saving resulting belief in a new belief object
					
					bel_used = [];
					for in = 1:size(bel_used,1)
						bel_used = [bel_used   num2str(list_bbf_unique(in,[1 3 4])'),','];
					end
					belief(ibel).origin = [bel_used' ones(length(bel_used),1)];
					belief(ibel).computed = expr;
					
					belief(ibel).short_name = answer{1};
					if isempty(answer{2})
						belief(ibel).full_name = answer{1};			
					else
						belief(ibel).full_name = answer{2};
					end
				
					belief(ibel).domain_variable = var_num_final ;
					belief(ibel).domain_cardinality = list_fr_card ;			
					
					[n remn] = words_for_set(prod(list_fr_card )) ;
					
					belief(ibel).bp(1).form(1).focal_sets = foc_final;
					belief(ibel).bp(1).form(1).values = m_final;
					
				else
					% the expr was erroneous, no bp's or logical operators wrongly ordered
					set(h_expr,'String',expr); % we put back what we had 
					
					hdw = warndlg(['Logical expression in the bottom text box not correct. ',...
					'You can delete the whole expression by selection/erase. '...
					'You can also type directly in the bottom text box, but bewrae of the syntax.']);
					uiwait(hdw);
					log_expr_wrong = 1;
					
				end % if expr_ok == 1
			
				if strcmp(which_combi ,'mixed')
					delete(h_expr)
				end % 
			
				text = 'Do you want to see the results now?';
				choice33 = text_with_n_buttons(calculator_handle, text, {'Yes','No'}, [1]);
				
				delete(calculator_handle);figure_data(ind_fig) = [];delete(hd_help)				
				if choice33 == 2
					go_on = 0;
				else
					% the entry to the bp_editor window
					run_belief_potential(ibel);					
					go_on = 0;
				end
			else % if choice19 == 2 cancel
				delete(calculator_handle);
				figure_data(ind_fig) = [];
				delete(hd_help)				
				go_on = 0;
			end  % if choice19 == 1 % from part_andor_dist_combi, I did not cancel
		end % if choice17 == 2 % cancel at first query display
	end	% while go_on
end % if ok
