function [out] = fmt(in, in_type, out_type, normalized)
% general function that computes any fast mobius transform

% between any of m, b, pl, q, wc, wd
% into any in the same set + betp.
% fourth parameter optional, produce the normalized outcome, 
% can be anything, its presence is enough, but 'n' is clever.
% in and out are column vectors

% note that you can also use directly the fmt functions by calling them
% by out = xtoy(in). It is up to the user. In that case in and out are ROW vector

%
%   Version : April 30,03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% for historical reasons these FMT function use line vecotrs and the TBMLAB use vertical vectors
% so I look if input vectors are row or column and transpose the column one
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[nl nc] = size(in);
if nl > nc % case column
	in = in';
	to_be_transp = 1;
else % case row
	to_be_transp = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

in_type = lower(in_type);
out_type = lower(out_type);

out = []; % just to avoir 'argument not assigned' when error

% we verify that the request is valid

% list of valid requests
v{1} = 'm'; v{2} = 'b'; v{3} = 'bel';v{4} = 'pl';v{5} = 'q'; v{6} = 'w';
v{7} = 'betp'; v{8} = 'wc'; v{9} = 'wd';
% w = wc
in1 = strmatch(in_type,v,'exact');
out1 = strmatch(out_type,v,'exact');

if length(in1) == 0|length(out1)==0
	'ACCIDENT in fmt: You have mistyped your request, one of the ..._types is not valid'
	fprintf('Your request was: %s %s \n',in_type, out_type)
else 
	% check validity of request with ismember
	lm = length(in);
	if in1 == 7
		natoms = lm;
	else
		natoms = round(log2(lm)); 
	end
	if 2^natoms == lm | in1 == 7 % so size is OK
		
		% wc is equal to w (historical reason, compatibility with past versions)
		if strcmp(in_type,'wc')
			in_type = 'w';
			in1 = 6;
		end
		if strcmp(out_type,'wc')
			out_type = 'w';
			out1 = 6;
		end
	
		% we first treat the betp request
		if out1 == 7
			if in1 == 1
				out = mtobetp(in);
			elseif in1 == 6 % wc case
				% for wc to betp I have a rounding error because == many combi and m() = 1
				out = wctobetp(in);
			else
				out = eval(strcat('mtobetp(',in_type,'tom(in))'));
			end		
		elseif in1 == 7 % I enter betp and need LC Isopign out_type
			[sel_m, mat_focal] = get_LCm_from_BetP(in', [pow2(0:natoms-1)]');
			% getting all masses
			m = zeros(1,2^natoms);
			m([mat_focal + 1]) = sel_m';
			
			if strcmp(out_type,'m')
				out = m; % not very clever request, asking what you had!
			else
				out = eval(strcat('mto',out_type,'(m)'));
			end
			if nargin == 4
				out = normalize(out, out_type, natoms);
			end					
		else		
			% so it is not a betp at in or out request	
			if strcmp(in_type,out_type)
				out = in; % not very clever request, asking what you had!
			else
				out = eval(strcat(in_type,'to',out_type,'(in)'));
			end
			if nargin == 4
				out = normalize(out, out_type, natoms);
			end			
		end
	else
		'ACCIDENT : the length of the in vector must be 2^nbr_atoms'
	end
end

[nl nc] = size(out);
if nl < nc % case column
	out = out';
end
