function [var_name_list, element_list, element_numbers,label_details] = ...
creating_elements(var_list_cdi)
% from several variables, prepare a list of elements
% Input is a list of variable numbers
% I build the elements of its product space

%
%   Version : June 1, 03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

% example var_list = [num_var1 num_var2], var1 = x1,x2,x3, var2 = y1,y2
% element_numbers = [ 1 1 ; 1 2 ; 1 3 ; 2 1 ; 2 2 ; 2 3]
% element_list{1:6} =  {'y1 x1 ', 'y1 x2 ', 'y1 x3 ', 'y2 x1 ', 'y2 x2 ', 'y2 x3 '} 
% var_name_list = ['var2 var1']  short_names
% label_details list where the label for each variable starts in a line of element_list
% label_details.text{i,j} = element j of frame of var i in list var_in
% label_details.length(i) = nbr_char to display var i;
%
% also used for continuous - integers other variables
% label_details.mem_index_contint = index_contint;
% label_details.mem_index_discr = index_discr;
% label_details.mem_num_var_contint = num_var_contint;
% label_details.mem_num_var_discr = num_var_discr;

% if card v1 = 3 and card v2 = 2, and domain_variable = v1 v2 (numbers)
% element_numbers = 
% 	v2 	v1
% 	1 	1
%	1	2
%	1	3
% 	2 	1
%	2	2
%	2	3
% element_list = the text for element in frame (suppose v1 ABC v2 NY)
% 	v2 	v1
% 	N 	A
%	N	B
%	N	C
% 	Y 	A
%	Y	B
%	Y	C
% var_name = v2 v1
% label_details.text{1,:} = A B C
% label_details.text{2,:} = N Y
% element j of frame of var i in list domain_variable reverse of var_name
% label_details.length(:) = nbr char var 1, nbr char var 2

% for integers and continuous, all that is accepted is a 'rectangle' for each element of the discrete space
% suppose V1 = Y/N, v2 = A/B/C v2 = Int, v4 = Cont
% there are 6 lines 
% 	v1  v2	Int  	Cont
%	Y	A	fr 	to	fr	to
%	Y	B	fr 	to	fr	to
%	Y	C	fr 	to	fr	to
%	N	A	fr 	to	fr	to
%	N	B	fr 	to	fr	to
%	N	C	fr 	to	fr	to

% more general would need to define the indicator fct on IntxCont for each of the six eleemnts.
% a subset is
% 	v1  v2	Int  	Cont
%	Y	A	4	7	11.5	21.3
%	N	B	9 	19	42.3	71.4
% more elaborated subsets will not be cnsidered in TBMLAB as in fact all we care is one var continuous
% in which case we get only one pair   fr   to.
% initially fr to are min and max

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global var_continuous_integers

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% separating discrete / Integers / continuous variables 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

var_name_list = [];
element_list = [];
element_numbers = [];
label_details = [];

[list_discr,list_continuous, list_integers] = type_variable(var_list_cdi);
% list_discr.num or list_discr.ind, etc....

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the discrete variables
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

var_list = list_discr.num;  % only the discrete one
nbr_var = length(var_list);

if nbr_var 
	ind_var = list_discr.ind; 
	ind_frame = pin_manipulation('frame','num2ind',[variable(ind_var).frame]);
	% the elemnts are in frame(ind_frame(i)).elements
	% elements is a strvcat matrix
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% for space reason I can only take a few characters
	% so I need to find for each var how many char are allowed
	for i = 1:nbr_var
		[nbr_elem(i) n_char(i)] = size( frame(ind_frame(i)).elements );
		var_names{i} = variable(ind_var(i)).short_name;
		nbr_char(i) = max(length(var_names{i}), n_char(i));
		% n_char real length var elements, 
		% nbr_char length as allowed for one line
	end
	tot_char = sum(nbr_char) + nbr_var; % how many char are needed with blank separators
	
	% fitting number of char by variable so that it fist ONE line
	max_nbr_char = 80; % check if we have room for 80 char on a line
	nn = max(nbr_char);
	
	while tot_char > max_nbr_char
		for i = 1:nbr_var
			nbr_char(i) = min(nbr_char(i),nn);
		end
		tot_char = sum(nbr_char);
		nn = nn - 1;
	end
	% length allowed established, it is nn
	
	var_name_list = '';
	% creating a matrix with element{i,j} = 'True' ... the labels of the elements
	for i = 1:nbr_var
		for j = 1:nbr_elem(i)		
			x = ddblank(frame(ind_frame(i)).elements(j,:));
			nn = min(nbr_char(i), length(x) );		
			el_var{i}(j,1:nn) = x(1:nn);
			n_blk_elem = max(0, nbr_char(i) - nn);
			if n_blk_elem > 0
				el_var{i}(j,nn+1:nn+n_blk_elem) = ' '; 
			end		
			label_details.text{i,j} = el_var{i}(j,:) ;
		end	
		label_details.length(i) = nbr_char(i);
		
		x = strcat(var_names{i});
		nn = min(nbr_char(i),length(x));
		var_names_short = x(1:nn);
		n_blk_elem = max(0, nbr_char(i) - nn);	
		if n_blk_elem > 0
			var_names_short(nn+1:nn+n_blk_elem) = ' '; 
		end		
		var_name_list = [var_names_short,' ',var_name_list];
	end
	% % % % var_name_list = [' ', var_name_list];
	
	% the names for the elements that can be displayed are in el_var
	% 
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% creating a matrix  element_numbers  like [ 1 1 ; 1 2 ; 1 3 ; 2 1 ; 2 2 ; 2 3 ]
	% indicating which element of which variable is on a given line
	element_numbers = [1: nbr_elem(1)]';
	if nbr_var > 1
		for i = 2:nbr_var
			dmat = [];
			for j = 1: nbr_elem(i)
				dmat = [dmat ; j*ones(length(element_numbers),1) element_numbers ];
			end
			element_numbers = dmat;
		end  
	end

	% Case var1 = TF, var2 = ABC
	% mat = [1 1 ; 1 2 ; 2 1 ; 2 2 ; 3 1 ; 3 2]
	% left = ABC, right = TF, thus line is  var2 var1, and var1 turns faster than var2
	
	% creating a matrix which line is a world in the new product space
	for i = 1:size(element_numbers,1)
		el = ''; 
		for j = 1:nbr_var
			el= [el, el_var{nbr_var+1-j}(element_numbers(i,j),:) ,' ']; 
			label_details.elem_label{i,j} = el_var{nbr_var+1-j}(element_numbers(i,j),:);
		end
		element_list{i} = el;
	end
else
	var_name_list = [];
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% handling integer and continuous var
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
var_continuous_integers = [];
kk = 0;
for i_type = 1:2
	switch i_type
	case 1 % continuous
		num_var = list_continuous.num;
		ind_var = list_continuous.ind;
		typevar = 'Continuous';
	case 2 % integers
		num_var = list_integers.num;
		ind_var = list_integers.ind;
		typevar = 'Integers';
	end		

	if length(num_var)
		for i = 1:length(ind_var)
			x = variable(ind_var(i)).short_name;
			ind_fra = pin_manipulation('frame','num2ind',variable(ind_var(i)).frame);
			x_min = frame(ind_fra).min;
			x_max = frame(ind_fra).max;
			switch i_type 
			case 1 % continuous
				str_x_min = num2str(fix(x_min));
				str_x_max = num2str(fix(x_max));
				nbr_int = max([length(str_x_min),length(str_x_max)]);
				nbr_dec = max(1,6-nbr_int);
				%  123456.1    12345.1  1234.12    1.23456  0.12345   .00001
				nbr_char = nbr_int + nbr_dec + 1;
				frt = ['%',num2str(nbr_char),'.',num2str(nbr_dec),'f'];
			case 2 % integers
				str_x_min = num2str(x_min);
				str_x_max = num2str(x_max);
				nbr_char = max([length(str_x_min),length(str_x_max)]);
				frt = ['%',num2str(nbr_char),'i'];
			end			
		
			long = 2*nbr_char + 1;
			nn = min(long,length(x));			
			var_names_short = x(1:nn);
			n_blk_elem = max(0, long - nn);	
			if n_blk_elem > 0
				var_names_short(nn+1:nn+n_blk_elem) = ' '; 
			end		
			var_name_list = [var_name_list,' ',var_names_short];
			
			% building edit area for entering numbers, (one-same for each disc line)

			% needed data = var_num, var_ind, type, min, max, frt
			kk = kk + 1;
			var_continuous_integers(kk).short_name = variable(ind_var(i)).short_name;
			var_continuous_integers(kk).num = num_var(i);
			var_continuous_integers(kk).ind = ind_var(i);
			var_continuous_integers(kk).type = typevar;
			var_continuous_integers(kk).min = x_min;
			var_continuous_integers(kk).max = x_max;
			var_continuous_integers(kk).frt = frt;			
		end % for i = 1:length(ind_var)	
	end % 	if length(num_var)
end % for i_type = 1:2
