function [out] = bbd_gauss_gamma(x1,x2,a,b,n,potential_list)
% computing bel, etc for g(u,v) = f(v)g(u)
% f(v) = gauss(a,b)
% g(u) = gamma(n)

% f(v:a,b) = (1/sqrt(2b)b) exp(- (x-a)^2/ 2b^2)    m = a, var = b^2
% g(u:n) = (1/(n-1)!) u^(n-1) exp(-u)    (n=1 == exp neg) m = var = n
% out are the values in the same order as the potential_list

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% bel([x1,x2]) = int u=0:(x1+x2)/2  int v=x1+u:x2-u   f(v) dv  g(u) du
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% one must squeeze the limits to avoid useless integrations 
% hence squeezing doma	in if too large and thus lot of 0's
% exp(-30) = E-14, thus (y-a)/b < 30    y < 30*b + a
if x1 >= a		% x2 > x1 >= a  (y-a)/b < 30    y < 30*b + a
	y1 = min(10*b+a,x1);
	y2 = min(10*b+a,x2);
elseif x2 > a  % x1 < a   
	y1 = max(-10*b+a,x1);
	y2 = min(10*b+a,x2);
else  % x1 < x2 <= 0      (a-y)/b < 30    y < - 30*b + a
	y1 = max(-10*b+a,x1);
	y2 = max(-10*b+a,x2);
end

out = [];
for i = 1:length(potential_list)	
	switch lower(potential_list{i})	
	case 'm'
		u = (x2-x1)/2; v = (x1+x2)/2; 
		new = (1/(sqrt(2*pi)*b))*exp(-.5*((v-a)/b)^2) * u^(n-1)*exp(-u)/gamma(n);
	case {'bel','b'}
		u1 = 0; u2 = (y2-y1)/2;
		new = quad('gauss_gamma',u1,u2,[],[],a,b,n,y1,y2,'in');
	case 'pl'
		new = quad('gauss_gamma',0,10*b+a,[],[],a,b,n,y1,y2,'out');
	case 'q'
		u1 = (y2-y1)/2; u2 = 30*b+a;
		new = quad('gauss_gamma',u1,u2,[],[],a,b,n,y1,y2,'out');	
	case 'betp'
		% NA
	end
	out = [out new];
end

