function outmatrix = collapsemat(inmatrix, value_column)
% COLLAPSEMAT purges repeated rows in 2D INMATRIX, leaves only one representative
% adds values in VALUE_COLUMN of removed rows into value of the representative.
% INMATRIX is 2D array. 
%   One column is designated as value column. 
%   Other columns are designated as address
% VALUE_COLUMN the number of value column by default the value column is the last one.
%   A number for VALUE_COLUMN greater than number of columns in INMATRIX means all columns are address

[nrows, ncols]=size(inmatrix);
if (nargin==1)
    value_column=ncols;
end
addonecol=0;                    % to record if a column is added
if (value_column <= ncols)      % there is a value column
    if (value_column < ncols)   % a valid value_column number
        if value_column == 1
            inmatrix = cat(2, inmatrix(:,[2:ncols]), inmatrix(:, 1));
        elseif value_column > 1
            inmatrix = cat(2, inmatrix(:,[1:value_column-1]), inmatrix(:, [value_column+1:ncols]), inmatrix(:,value_column));
        end
    end
elseif (value_column > ncols)   % no value column
    addcolumn=ones(nrows, 1);
    inmatrix=cat(2, inmatrix, addcolumn);
    [nrows, ncols]=size(inmatrix);  % new size
    addonecol=1;
end

if nrows>1
    inmatrix=sortrows(inmatrix);
    begin_row=1;
    next_row=2;
    while next_row<=nrows
        if isequal(inmatrix(begin_row,1:ncols-1), inmatrix(next_row,1:ncols-1))
            inmatrix(begin_row,ncols)=inmatrix(begin_row,ncols)+inmatrix(next_row,ncols);     %take cummulative mass
            inmatrix(next_row,ncols)=0;
            next_row=next_row+1;
        else
            begin_row=next_row;
            next_row=next_row+1;
        end
    end
end

rowtoremove=find(inmatrix(:,ncols)==0);        % purge repeated rows
if ~isempty(rowtoremove)
    inmatrix(rowtoremove,:)=[];
end

if addonecol==1
    inmatrix(:,ncols)=[];
end
outmatrix = inmatrix;
%%%%%%%%%%%%%%% end of COLLAPSEMAT