function bel_idnum=solve(list_bels, list_vars)
%SOLVE calculate the maginal belief on the list of vars from the beliefs given in the list_bels
% LIST_BELS is a list of beliefs relevant to the calculation (input)
% LIST_VARS is a list of variables
% BEL_IDNUM is the id_number of marginal belief

global BELIEF QUERY;       % VARIABLE MOFI ATTRIBUTE STRUCTURE FRAME BELTRACE;

if nargin==1                % default case: all beliefs in the database are relevant
    list_bels=[BELIEF(:).number];
end

varbel=varbelcross(list_bels);          % take var-bel information from database
joindomain=varbel.varnums;              % all variables relevant
if isempty(intersect(joindomain, list_vars))
    fprintf('Invalid request: variable to solve is not in the domain.');
    bel_idnum = 0;                      % exception - not a real belief number
    return
end
vartodelete=setdiff(joindomain, list_vars); % the list of vars to be deleted
if ~isempty(vartodelete)
	vartodelete=chooseaseq(varbel, vartodelete);      % choose a deletion sequence
	nofvars=length(vartodelete);
	
	keybels=zeros(1,nofvars);               % to store the belief_numbers that obtained by removing a var
	for i=1:nofvars
        deletevar=vartodelete(i);
        varbel=removeavar(varbel, deletevar);
        keybels(i)=varbel.belnums(end);     % fprintf('keybels(%2d) = %2d\n', i, keybels(i));
	end
end

beltocombine=varbel.belnums;        % beliefs need to combine before the final report
% fprintf('List of bels to combine after removal = [%s]\n', num2str(beltocombine));
bel_idnum=multcombine(beltocombine);

% create a QUERY record
if isempty(QUERY)
    qcount = 0;
    qmaxnumber = 0;
else
    qcount = length(QUERY);
    qmaxnumber = max([QUERY(:).number]);
end

qcount = qcount+1;
qmaxnumber=qmaxnumber+1;

if ~isempty(vartodelete)
    deleteseqence=vartodelete;
	QUERY(qcount).number = qmaxnumber;
	QUERY(qcount).qvars = list_vars;
	QUERY(qcount).qbelbase = list_bels;
	QUERY(qcount).deleteseq=deleteseqence;
	QUERY(qcount).keybelief=keybels;
    QUERY(qcount).finalmarg=bel_idnum;
end


%%%%%%%%%%%%%% end of SOLVE
