function gui_position_setting
% all positions needed for displaying gui objects
% always in normalized (in 0, 1), except if concerns figures
% always 4 values
% gui_position(4,k)

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global gui_position  
global screen_size

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 1 = Help figure   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% fixing adapted width and in between   
switch computer
case 'MAC2'
	left_pos     =   8; % left most x position (also at right)
	horiz_shift  =   8;	% horizontal shift between 2 figures
	bottom_shift =  50; % I want to get bottom free for a comment window
	top_shift    =  50;	% I also need some room atop to avoid hidden parts with PC
% 	height_com	 =  80; % height of the comments instruction window
% 	vert_shift   =  32; % sapcing between top lower and bottom upper figures
	
case 'MAC'
	left_pos     =   3; % left most x position (also at right)
	horiz_shift  =   1;	% horizontal shift between 2 figures
	bottom_shift =  55; % I want to get bottom free for a comment window
	top_shift    =  75;	% I also need some room atop to avoid hidden parts with PC
% 	height_com	 =  80; % height of the comments instruction window
% 	vert_shift   =  32; % sapcing between top lower and bottom upper figures

case 'PCWIN'
	left_pos     =   10; % left most x position (also at right)
	horiz_shift  =    5; % horizontal shift between 2 figures
	bottom_shift =   50; % I want to get bottom free for a comment window
	top_shift    =   40; % I also need some room atop to avoid hidden parts with PC
% 	height_com  =    80; % height of the comments instruction window
% 	vert_shift   =   30; % sapcing between top lower and bottom upper figures
	
otherwise
	left_pos     =    3; % left most x position (also at right)
	horiz_shift  =    1; % horizontal shift between 2 figures
	bottom_shift =   50; % I want to get bottom free for a comment window
	top_shift    =   55; % I also need some room atop to avoid hidden parts with PC
% 	height_com  =    80; % height of the comments instruction window
% 	vert_shift   =   30; % sapcing between top lower and bottom upper figures
end	

if  screen_size(3)> 1024 % large Mac powerbook
	proportion   = .30; % propostion for UL figure
else % 1024 = classical today screen
	proportion   = .35; % propostion for UL figure
end


% the four values are for the x paramenters of the middle and bottom windows
% the [left width left width] of left right figures
x_pos_middle = [left_pos   ...
(proportion*screen_size(3)-left_pos-horiz_shift) ...
(proportion*screen_size(3) + horiz_shift)   ...
((1-proportion)*screen_size(3)-2*horiz_shift-left_pos)  ];

x_pos_botttom =     [left_pos  ...
(.5*screen_size(3) - horiz_shift-left_pos) ...
(.5*screen_size(3) + horiz_shift)...
(.5*screen_size(3) - 2*horiz_shift)-left_pos ];

% the four values are for the y paramenters of the middle and bottom windows	

y_pos(1:4) = [bottom_shift   0  ...
(bottom_shift)...
(screen_size(4) - bottom_shift - top_shift)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 1 = the Root Window in UL
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,1) = [x_pos_middle(1) y_pos(3)  x_pos_middle(2) y_pos(4)]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 2 = the TITLE in Blue, in UL
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

little_h_sh = .005; % horiz shift
little_v_sh = .005; % shift from top
title_h = .05; 	% where I put nice title for the whole session
help_width = .99; 
gui_position(:,2) = [little_h_sh   1-little_v_sh-title_h    help_width  title_h]' ;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 3 = available position for displaying helps Root Window UL
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

button_height = .05;  % space for the button at bottom
gui_position(:,3) = [little_h_sh   button_height   help_width ...
1-title_h-little_v_sh-button_height ]';	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 4 = OK button at bottom Root Window
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

button_height = .04;
button_width =  .15;
gui_position(:,4) = [ .50-button_width/2  .002     button_width  button_height]';		

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 5 = .... Command Window figure at UR
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,5) = [x_pos_middle(3) y_pos(3)  x_pos_middle(4) y_pos(4)]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 6 = left ListBox, we can accomodate only 3 of them : see 6, 7, 12
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,6) = [.35 .10 .20 .89]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 7 = middle ListBox, we can accomodate only 3 of them : see 6, 7, 12
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,7) = [.57 .10 .20 .89]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 8 = grenn button centger Root Window
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,8) = [.20 .45 .60 .10]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 11 = In dialog frame/show_me_elements, where elements are detailled 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,11) = [.005 .005 .25 .30]' ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 12 = right ListBox, we can accomodate only 3 of them : see 6, 7, 12
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,12) = [.79 .10 .20 .89]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 13 = LL frame-text-button in current figure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,13) = [.01 .01 .30 .20]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 14 = full figure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,14) = [.001 .001 .999 .999]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 15 = UR a query box filled with click in data (for get bp with logical)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,15) = [.003 .003 .746 .196]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 16 = UR a query box filled with a mass (for get bp with logical)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,16) = [.76 .001 .20 .10]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 17 = UR a list box with possible values for frame (for get bp with logical)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,17) = [.30 .20 .25 .795]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 18 = UR a red button for getting possible values in multiselection (for get bp with logical)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,18) = [.001 .15 .10 .05]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 19 = full screen road map of all objects
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,19) = [x_pos_botttom(1) y_pos(3) ...
screen_size(3)-x_pos_botttom(1)-2*horiz_shift  y_pos(4)]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 20 = a window in the middle of everything for short display only
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,20) = [50 200   .75*screen_size(3)   screen_size(4)-300]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 21 = UR a query box filled with a mass (for get bp with logical)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,21) = [.762 .003 .096 .046]';

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% 22 = UR a text box asking mass (for get bp with logical)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

gui_position(:,22) = [.76 .051 .235 .08]';
