function best_supported_set(ind_bel,ind_pot)
% finding the best supported subset a la Appriou
%

%
%   Version : Apr 10, 03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief
global flag_str  screen_size max_card_fmt
global Which_Courier 
global gui_position  figure_data  flags  help_level help_label

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% fontsize for Which_Courier
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch computer
case 'MAC2'
	ftsz = 10;
otherwise
	ftsz = 9;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% finding the form
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

form_ind_list = find(strcmp({belief(ind_bel).bp(ind_pot).form(:).type},'m'));

switch length(form_ind_list)
case 0
	% no m 
	hdw = warndlg('There is no m function for this potential. I skip request.')
	uiwiat(hdw);
	return
case 1
	ind_for = form_ind_list;
otherwise
	% several m, select one
	hdw = warndlg('There is several m functions for this potential. Not correct. I skip request.')
	uiwiat(hdw);
	return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting bba and focal sets
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m = belief(ind_bel).bp(ind_pot).form(ind_for).values;
foset = belief(ind_bel).bp(ind_pot).form(ind_for).focal_sets;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building the labels for the worlds of the prodcut space, ready for display
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[var_name_list, element_list, element_numbers,useless] = creating_elements (belief(ind_bel).domain_variable);
[cardinal useless] = size(element_numbers);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% ontrolling feasability (not too many elements)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

too_many_elements = 50;
if cardinal > too_many_elements
	hdw = warndlg(['There are too many elements for the present version. I skip.',...
	' I accept 50 elements. Cardinal = ',num2str(cardinal)]);
	uiwait(hdw);
	return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the figure
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

scrsize = screen_size;
height_window = scrsize(4)-100;
width_window = scrsize(3)-20;
pos_fig = [10 50  width_window height_window];
hd_fig = figure( ...
'Units','pixels',...
'Color',[1 1 1], ...
'name','Best Supported Set.');
set(gcf,'Position',pos_fig);
ind_fig = length(figure_data) + 1;
figure_data(ind_fig).name = 'Best Supported Set.';
figure_data(ind_fig).handle = hd_fig;
figure_data(ind_fig).position = pos_fig;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% put its menus
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

k=0;
k=k+1;make_fig_menu(ind_fig,k,'File',0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Quit',1,'general_clb(1)','on');
k=k+1;make_fig_menu(ind_fig,k, help_label,0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Helps On', 3,'general_clb(2)','on');

menu_manager('create',ind_fig)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The BSS algorithm
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

small_cardinal = max_card_fmt;

if cardinal <= small_cardinal
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% using the brute force (all pl) with FMT
	% this may have to be cahnged for large sets
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% % % % 	if strcmp({belief(ind_bel).bp(ind_pot).form(:).representation},'ordered')
	% % % % 		m_vect = m;
	% % % % 	else
	% % % % 		m_vect = zeros(1,pow2(cardinal));
	% % % % 		for i = 1: length(m)
	% % % % 			m_vect(foset(i)+1) = m(i);
	% % % % 		end
	% % % % 	end
	% % % % 
	% % % % 	pl = fmt(m_vect,'m','pl','n');
	% % % % 	% get normalized pl from m, row vector
	
	% get Pl (normalized), column vector, 
	[computed_forms,uncomputed_forms, error_mode] = transform_bp(ind_bel,ind_pot,{'Pl'})	
	ind_for = find(strcmp({belief(ind_bel).bp(ind_pot).form(:).type},'Pl'));
	pl = belief(ind_bel).bp(ind_pot).form(ind_for).values;
	
	% card of each element of vector pl
	card = [0];
	for i = 1:cardinal
		card = [card card+1];
	end
	% get the max_pl for each cardinality and its indexes (>1 possible)
	for n = 1:cardinal		
		max_pl(n) = max([pl(find(card == n))]);	
		% I decide only ONE per cardinal
		ind_max_pl{n} = intersect(find(pl== max_pl(n)),find(card == n)); % maybe several = max_pl
		ind = ind_max_pl{n}(1);
		foc_card(n,1) = ind-1; % focal set of this pl (always < 50 elements so long number in 1 word)
	end
	% outcome = max_pl(n),foc_card(n,:) n = 1:cardinal
	
else	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% using the stepwise procedure
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	% we work with un_normalizing the masses (end result will be normalized with pl(universe))
	
	% starting the procedure: finding the best supported singletons
	for i = 1:cardinal
		foc(i,:) = long_onbit2num(i,cardinal); 
		pl(i) = 0;
		for j = 1:length(m)
			if bitand(foc(i,:),foset(j,:)) ~= 0
				pl(i) = pl(i) + m(j);
			end
		end
	end
	max_pl(1) = max([pl]);
	index = find(pl== max_pl(1));
	ind = index(1);
	foc_card(1,:) = foc(ind,:);
	clear foc pl
	
	for n_card = 2:cardinal
		bits_on = long_num2onbit(foc_card(n_card-1,:),cardinal);
		ik = 0;
		for i = 1:cardinal
			ik = ik + 1;
			if ~ismember(i,bits_on)
				foc(ik,:) = bitor([foc_card(n_card-1,:)],[long_onbit2num(i,cardinal)]);
				pl(ik) = 0;
				for j = 1:length(m)
					if bitand(foc(ik,:),foset(j,:)) ~= 0
						pl(ik) = pl(ik) + m(j);
					end
				end	
			end
		end
		max_pl(n_card) = max([pl]);
		index = find(pl== max_pl(n_card));
		foc_card(n_card,:) = foc(index(1),:);
		clear foc pl			
	end	

	max_pl = max_pl/max_pl(cardinal); % normalized
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% listing the elements at UL, buiulding the matrix with results
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% building many_blanks (the left space for UL part) and separator (the horizontal line every 5 lines)
nbr_sets = cardinal;
separator = '';
many_blanks = '';
for i = 1:length(var_name_list)
	many_blanks = [many_blanks,' '];
	separator = [separator,'-'];
end
many_blanks = [many_blanks,'|'];
separator = [separator,'|'];

for i = 1:nbr_sets
	if i == 5*fix(i/5) 
		separator = [separator,'----|'];
	else
		separator = [separator,'-----'];
	end
end

% line{1}
line{1} = [var_name_list,'|'];
for i = 1:nbr_sets
	if i == 5*fix(i/5) 
		line{1} = [line{1},sprintf('%4i',[i]),'|'];
	else
		line{1} = [line{1},sprintf('%4i',[i]),' '];
	end
end

%line{2}
line{2} = many_blanks;
if length(many_blanks) <= 3
	line{2}(1) = 'P';
	line{2}(2) = 'l';
else
	line{2}(1) = 'P';
	line{2}(2) = 'l';
	line{2}(3) = '-';
	line{2}(4) = '>';
end

%line{3}
line{3} = separator;

%  next lines and finishing line{2} for pl
k = 3; 
for n = 1:cardinal
	
	% preparing the other lines (still miss the x)
	if n > 1 & n-5*fix(n/5) == 1
		k=k+1;line{k} = separator;
	end
	k=k+1;line{k} = [element_list{n},'|'];
	
	% adding pl to line{2}
	if n == 5*fix(n/5) 
		line{2} = [line{2},sprintf('%4.2f',max_pl(n)),'|'];
	else
		line{2} = [line{2},sprintf('%4.2f',max_pl(n)),' '];	
	end

end
% all lines are built

% bnuilding the dots describing the sets
for n = 1:cardinal % form left ot right, one column = the pl max
	% thus for each vertical line
	il = 3;
	bits = long_num2bit(foc_card(n,:), cardinal);
	
	for j = 1:cardinal % going down
		il = il + 1;
		if j > 1 & j-1 == 5*fix(j/5)
			il = il + 1;
		end
		if bits(j) == 1 
			line{il} = [line{il}, '  x '];
		else
			line{il} = [line{il}, '    '];
		end
		if n == 5*fix(n/5)
			line{il} = [line{il},'|'];
		else
			line{il} = [line{il},' '];
		end								
	end		
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% plotting the list of elements and selected sets with pl   units in pixels	
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pos_line = pos_fig;
hd(2) = uicontrol(gcf,'units','pixels',...
'fontname',Which_Courier,...
'fontsize',ftsz,...
'position',pos_line,...
'style','listbox',...
'string',line); % the data

[out_text, ht_text] = textwrap(hd(2),line);

ht_text(3) = min(ht_text(3) + 25,450) ; % because of the right scale and I limit its width
ht_text(4) = min(ht_text(4)+50,pos_fig(4)-50) ; % and it height

delete(hd(2));
pos_line = [5    pos_fig(4)-ht_text(4)-20  ht_text(3)+10   ht_text(4)+15];
hd(1) = uicontrol(gcf,'units','pixels',...
'fontname',Which_Courier,...
'fontsize',ftsz,...
'position',pos_line,...
'style','frame'); % the frame

pos_line = pos_line + [5 5 -10 -10];
hd(2) = uicontrol(gcf,'units','pixels',...
'fontname',Which_Courier,...
'fontsize',ftsz,...
'position',pos_line,...
'style','listbox',...
'horizontalalignment','left',...
'string',line); % the data

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% finding the best supported set for r =  [ 0 .2 .4 .6 .8 1 ]   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
r_val = [0 .2 .4 .6 .8];

% data to be plotted
for n = 1:cardinal
	x_mat(n) = n;
	k = 0;
	for r = r_val
		k=k+1;y_mat(k,n) = max_pl(n)/(n^r);
	end
end

% r = 1 for universe, r=0 for singletons
for i = 1:length(r_val)
	[pl1 pos1] = max(y_mat(i,:));
	if pos1 == 1| pos1 == cardinal
		val(i) = 0; % useless
	else
		val(i) = 1; % nice 'bell'
	end
end
ind_val = find(val == 1);
if isempty(ind_val)
	r_val = [0 .2 .4 .6 .8];
else
	coef1 = max(r_val(ind_val(1))-.2,0);
	coef2 = min(r_val(ind_val(length(ind_val)))+.2,1);
	delta = (coef2 - coef1)/4;
	r_val = [coef1 coef1+delta coef1+2*delta coef1+3*delta coef1+4*delta];
end	

% data to be plotted
for n = 1:cardinal
	x_mat(n) = n;
	k = 0;
	for r = r_val
		k=k+1;y_mat(k,n) = max_pl(n)/(n^r);
	end
end

txt{1} = 'For r, cardinal best set'; 
for k = 1:5
	[mx loc_mx] = max([y_mat(k,:)]);
	txt{k+1} = [sprintf('%5.2f',r_val(k)), '      ',sprintf('%3i',loc_mx)];
end

hd_r = uicontrol('units','pixels',...
'fontname',Which_Courier,...
'fontsize',ftsz,...
'position',[5 5 300 500]);
[useless,ht] = textwrap(hd_r,txt);
delete(hd_r)

pos_hdr = [5 5 ht(3)+20 ht(4)+20];
hd_r(1) = uicontrol('units','pixels',...
'fontname',Which_Courier,...
'fontsize',ftsz,...
'position',pos_hdr,...
'style','frame');

pos_hdr = pos_hdr + [5 5 -10 -10];
hd_r(2) = uicontrol('units','pixels',...
'fontname',Which_Courier,...
'fontsize',ftsz,...
'position',pos_hdr,...
'style','text',...
'HorizontalAlignment','left',...
'string',txt);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% plotting criteria for r =  r_val
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% plotting
pos_gr = [pos_line(1)+pos_line(3)+105    105   ...
pos_fig(3)-(pos_line(1)+pos_line(3)+5)-155	pos_fig(4)-210];	

hd_plot(1) = axes('parent',gcf,...
'units','pixels',...
'position',pos_gr,...
'Box','on');

hd_lines1{1} = plot(x_mat',y_mat');

title('Best Supported Set');	
ylabel('pl(A)/|A|\^r');
xlabel('Sets: refer to the column number (see top line) of list at right.');
text_for_plot = [];
for i = 1:length(r_val)
	text_for_plot = [text_for_plot; 'r=',sprintf('%4.2f',r_val(i))];
end
text((nbr_sets+.1)*ones(5,1),[y_mat(:,nbr_sets)],text_for_plot);
set(gca,'Layer','top', 'YTick',[0:.1:1], 'Xlim',[1 nbr_sets+2],'Ylim',[0 1]);

set(gca,'Ygrid','on','Xgrid','on')
drawnow	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% quitting or getting helps   
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
go_on = 1;
while go_on
	uiwait(gcf)	
	switch flags(1) 
	case 1 % quit
		delete(gcf)
		figure_data(ind_fig) = [];
		go_on = 0;
	case 2 % help with button return
		hd_help = forced_display_helps(1,45);
		delete(hd_help);
		figure(hd_fig);
	end
end % while go on
