function [I,S,xmin,xmax]=dubois(b,B,a,c,d)

% computation of the min and max of a'*x under the constraints
%   c'*x=d
%   b <= x <= B

% b, B, a, c are column vectors of size (n,1)
% d is a scalar

% Outputs: 

% xmin is the (n,1) vector solution of the minimization problem
% I = min a'x  and I=a' xmin

% xmax is the (n,1) vector solution of the maximaization problem
% S = max a'x  and S=a' xmax


% From the paper :
% D. Dubois and H. Prade. Addition of interactive fuzzy numbers. IEEE Transactions on 
% Automatic Control, 26(4):926--936, 1981.


n=length(a);

a = d * a./c;
b = c.*b/d;
B = c.*B/d;

[a1,i1]=sort(a);
b1=b(i1);
B1=B(i1);

I=realmin;
S=realmax;

B1a1=B1.*a1;
b1a1=b1.*a1;

xmin=-ones(size(a));xmax=xmin;
for k=1:n,
  x1 = [B1(1:k-1);(1-sum(B1(1:k-1))-sum(b1(k+1:n)));b1(k+1:n)];
  x2 = [b1(1:k-1);(1-sum(b1(1:k-1))-sum(B1(k+1:n)));B1(k+1:n)];
  X = sum(B1a1(1:k-1)) + (1-sum(B1(1:k-1)) ...
      - sum(b1(k+1:n)))*a1(k) + sum(b1a1(k+1:n));
  Y = sum(b1a1(1:k-1)) + (1-sum(b1(1:k-1)) ...
      - sum(B1(k+1:n)))*a1(k) + sum(B1a1(k+1:n));
  admiss1 = (length(find((x1>= b1-eps)&(x1 <= B1+eps)))==n);
  admiss2 = (length(find((x2>= b1-eps)&(x2 <= B1+eps)))==n);
  if (X>=I) & admiss1, 
   I=X;
   xmin(i1)=x1;
  end;
  if (Y<=S) & admiss2,
   S=Y;
   xmax(i1)=x2;
   end;
end;


