function error_message=lcp_const(selval_ind)

% Finds the least committed belief function compatible with a set of constraints

% "Least committed" is defined as maximizing the cardinality 
%
% card(m)=sum_A m(A)|A|
%
% The constraints are m, bel, pl or betP values or intervals for selected subsets of 
% the frame (all the values or intervals must be of the same type)
% If the constraints are on m, then by default m(A)=0 for all subsets A for which no values 
% are provided (i.e., we supply the focal elements).
% For other types of constraints, bel-pl-BetP(A) in [0,1] by default

% Handling of type 'bel', 'pl' and 'betp' constraints requires the Optimization toolbox
% and possible only for frame size <= 12.

% selval_ind is an index of a "selvalbp" object, with the following fields:

% selvalbp(selval_ind).belief       : index of belief
% selvalbp(selval_ind).focalsets    : matrix of selected focal sets in long number format
% selvalbp(selval_ind).val          : values of intervals for selected focal sets
% selvalbp(selval_ind).type                     : 'm', 'bel', 'pl' or 'betp'

% The program fills in the following fields:

%    belief(bel_ind).bp(1).form(1).type
%    belief(bel_ind).bp(1).form(1).focal_sets
%    belief(bel_ind).bp(1).form(1).values

% where bel_ind is the value of selvalbp(selval_ind).belief

global belief selvalbp
global nbr_bit_per_word
global BelfrM PlfrM BetPfrM     % should be declared in calling program. Avoids recalculation
% if already computed

bel_ind=selvalbp(selval_ind).belief;        % the index of the belief function
card = belief(bel_ind).domain_cardinality;  % the cardinal of the frame
focals_num=selvalbp(selval_ind).focalsets;  % the focal sets in long number form
values=selvalbp(selval_ind).val;            % the values (m, bel,pl or betp)
[n,r]=size(values);                         % n=number of values, r=1 (numbers) or 2 (intervals)

if selvalbp(selval_ind).type == 'm',
    % The simplest case: masses
    cardf=sum(focals_num,2);     % cardinalities of the focal elements
    % (needed to compute the cardinality of the bbm)
    if r==1, values=[values values]; end        % we translate in interval format
		if (sum(values(:,1))>1) | (sum(values(:,2))<1),
			error_message='Constraints are inconsistent. No solution exists.';
			return
		end;
		[tmp1,tmp2,tmp3,bbm]=dubois(values(:,1),values(:,2),cardf,ones(n,1),1);    
		% the optimization routine
		ii=find(bbm>1e-6); % finds the focal elements of the resulting bbm
		belief(bel_ind).bp(1).form(1).type='m';
		belief(bel_ind).bp(1).form(1).focal_sets= [focals_num(ii)]';
		belief(bel_ind).bp(1).form(1).values=[bbm(ii)]';
		belief(belief_index).bp(ind_bp).form(1).representation = 'selected'; 
		belief(belief_index).bp(ind_bp).form(1).foset_meaning = set_meaning; 
	else,
		% % % % %     s=ver('optim');
		
		
		
		s.Version = 1;
		
		
		
		
		
		
		
		if isempty(s),
			error_message='Optimization toolbox non available.';
			return;
		end;
		if card > 12,
			error_message='Problem is too big. Size of frame > 12.';
			return;
		end;
		nf=2^card;      % the total number of subsets
		cardA=[0;1];
		for i=2:card,
			cardA=[cardA;1+cardA];          % vector with cardinalities of all subsets
		end;
	
		all_subsets=(0:nf-1)';                        % all the subsets in long_num form
		
		switch selvalbp(selval_ind).type
		case 'bel',
			% Construction of the matrix for converting m into bel
			if ~exist('BelfrM') | (size(BelfrM,1) ~= nf),      % checks if BelfrM exists with correct size             
				BelfrM=[1];
				for i=1:card,
					BelfrM=[BelfrM zeros(2^(i-1));BelfrM BelfrM];
				end;
				BelfrM(:,1)=zeros(nf,1);
			end;
			R=BelfrM;
			belief(bel_ind).bp(1).form(1).type='bel';
		case 'pl',
			% Construction of the matrix for converting m into pl
			if ~exist('PlfrM') | (size(PlfrM,1) ~= nf),      % checks if PlfrM exists with correct size             
				BfrM=[1];
				for i=1:card,
					BfrM=[BfrM zeros(2^(i-1));BfrM BfrM];
				end;
				I=eye(nf);
				J=I(:,nf:-1:1);
				PlfrM=1-J*BfrM;
			end;
			R=PlfrM;
			belief(bel_ind).bp(1).form(1).type='pl';
			
		case 'betp',
			% Construction of the matrix for converting m into BetP
			if ~exist('BetPfrM') | (size(BetPfrM,1) ~= nf),      % checks if BetPfrM exists with correct size             
				cardAB=[0 0;0 1];
				for i=2:card,
					cardAB=[cardAB cardAB;cardAB 1+cardAB];
				end;
				cardAmoins=[0;1./cardA(2:end)];
				BetPfrM=cardAB*diag(cardAmoins);
			end;
			R=BetPfrM;
			belief(bel_ind).bp(1).form(1).type='betp';
			
		end;
	
		if s.Version>=2,    % For version 2 of optimization toolbox
			if r==2,
				A=[R(focals_num+1,:);-R(focals_num+1,:)];
				b=[values(:,2);-values(:,1)];
				Aeq=ones(1,nf);
				beq=1;
			else, % r=1, it is better to encode as equality constraints
				A=[];
				b=[];
				Aeq=[R(focals_num+1,:);ones(1,nf)];
				beq=[values;1];
			end;
			lb=zeros(nf,1);
			ub=ones(nf,1);
			
			
			disp('j''ai rien a foutre ici !!!!!!!!!!!!!!!')
% 			[bbm, tmp, exitflag] = linprog(-cardA,A,b,Aeq,beq,lb,ub);
			
			if exitflag < 0,
				error_message='Constraints are inconsistent. No solution exists.';
				return
			end;
		else,               % For version 1 of optimization toolbox
			if r==2,
				A=[ ones(1,nf);
                R(focals_num+1,:);
                -R(focals_num+1,:)];
				b=[1;
                values(:,2);
                -values(:,1)];
				Neq=1;
			else, % r=1, it is better to encode as equality constraints
				A=[R(focals_num+1,:);ones(1,nf)];
				b=[values;1];
				Neq=n;
			end;  % if r==2
			lb=zeros(nf,1);
			ub=ones(nf,1);
			
			
			
			
			
			%%%%%%% plug for me  XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
			[bbm,tmp,how]=hklp(-cardA,A,b,lb,ub,[],Neq);
			
			
			
			
			
			
			if ~(how=='ok'),
				error_message='Constraints are inconsistent. No solution exists.';
				return
			end;    
		end;   % if s.Version>=2,
	
		ii=find(bbm>1e-6);                          % finds the focal elements of the resulting bbm
		belief(bel_ind).bp(1).form(1).focal_sets=[all_subsets(ii)];
		belief(bel_ind).bp(1).form(1).values=[bbm(ii)];
	end;

	error_message='ok';
	
	% NOT ok if more 50 elements, but may not be used in that case
	% focal set masses = column vectors
	% 	belief(belief_index).bp(ind_bp).form(2).type = the_type; 
	% 	belief(belief_index).bp(ind_bp).form(2).representation = 'selected'; 
	% 	belief(belief_index).bp(ind_bp).form(2).focal_sets = st_focal;
	% 	belief(belief_index).bp(ind_bp).form(2).values = st_mass;
	% 	belief(belief_index).bp(ind_bp).form(2).foset_meaning = set_meaning; 
	
% use belief(bel_ind).bp(1).form(1).focal_sets and not belief(bel_ind).bp(1).form(1).focalsets
