function pack_mass(ind_b,ind_p,ind_f)
% packing a set of masses so that every focal set is different

%
%   Version : Apr 1,03.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global belief

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% adding masses given to same focal sets
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

% mass m(i) given to foc_new(i,:)
% there are duplicate in foc_new(i,:)

eval('mass = belief(ind_b).bp(ind_p).form(ind_f).values;ok_eval = 1;','ok_eval=0;');

if ok_eval == 0
	hdw = warndlg('I cannot pack. Programming error. Request non exiting field.');
	uiwait(hdw);
	return
else
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% I fill every mass with a logical expression
	%
	if isfield(belief(ind_b).bp(ind_p).form(ind_f),'log_expr')
		lg_expr = belief(ind_b).bp(ind_p).form(ind_f).log_expr;
		% making sure all log expr exist
		if length(lg_expr)< length(mass)
			lg_expr{length(mass)} = '';
		end
	else
		lg_expr{length(mass)} = ''; % everything missing is ''
	end
	%
	% all log expr OK
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% deleting mass = 0
	%
	del_ind = find(mass == 0)';
	if length(del_ind)
		% some 0 masses
		belief(ind_b).bp(ind_p).form(ind_f).values(del_ind) = [];
		belief(ind_b).bp(ind_p).form(ind_f).focal_sets(del_ind,:) = [];
		lg_expr(del_ind) = '';
		if isfield(belief(ind_b).bp(ind_p).form(ind_f),'contint') 
			for ijk = 1:length(belief(ind_b).bp(ind_p).form(ind_f).contint)
				if ismember(ijk, del_ind)
					belief(ind_b).bp(ind_p).form(ind_f).contint(del_ind) = [];
				end
			end
		end
	end
	%
	% all 0 masses deleted
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% BEG first sort without regard to cont/int var
	%	
	focal = belief(ind_b).bp(ind_p).form(ind_f).focal_sets;
	mass = belief(ind_b).bp(ind_p).form(ind_f).values;
	
	[sorted_data,ndx] = sortrows([focal mass]);
	[nr nc] = size(sorted_data);
	nbr_col_foset = nc-1;
	
	mm = sorted_data(:,nc);
	
	indx = [0 find(sum(abs(diff(sorted_data(:,1:nc-1),1,1)),2))' nr];
	
	% these are the position where focal sets change
	% with sorted_data = 
	%    var2  var1
	%      1     3
	%      1     3
	%      1     3
	%      2     1
	%      2     4
	%      2     4
	% we get indx = [0 3 4 6], so we must add mm(1:3) mm(4:4) mm(5:6)
	
	% creating an extra index to verify if two equal discr foset share same cont/int limits
	ind_contint = [];
	if isfield(belief(ind_b),'dci_var')
		if isfield(belief(ind_b).dci_var,'pin_c') & size(belief(ind_b).dci_var.pin_c,1) == 3 
			ind_contint = belief(ind_b).dci_var.pin_c(3,:) ; 
		end
		if isfield(belief(ind_b).dci_var,'pin_i') & size(belief(ind_b).dci_var.pin_i,1) == 3
			ind_contint = [ind_contint    belief(ind_b).dci_var.pin_i(3,:)];
		end
	end
	%
	% END first sort without regard to cont/int var
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% BEG handling contint case
	%	
	if length(ind_contint)
		% there are some cont/int var
		nc = nc + 1;
		sorted_data(:,nc) = sorted_data(:,nc-1); % I put mass to the right
		sorted_data(:,nc-1) = ndx; % I add a column with 1, 2, 3... as if all contint limits were diff		 
		
		for i = 1:length(indx)-1 
			% for every case where equality of foset is possible
			if indx(i+1) > indx(i)+1
				% possible equality, at least 2 with same discretes variables				
				limit_contint = [];
				for j = indx(i)+1:indx(i+1)-1
					limit_mem = belief(ind_b).bp(ind_p).form(ind_f).contint(ndx(j)).limits; 
					ii = 0;
					for list_j = j+1:indx(i+1)
						ii = ii + 1;
						limit_contint{ii} = belief(ind_b).bp(ind_p).form(ind_f).contint(ndx(list_j)).limits; 
					end
					index = find_fs_list(1,ones(indx(i+1)-j-1,1),limit_mem,limit_contint);
					if index
						% there is an equality and I change index of nc-1 with number last equal
						sorted_data(j,nc-1) = j + index;
					end
				end	
			end
		end
	
		% sorting the extended set of data
		[sorted_data,ndx] = sortrows(sorted_data);
		% saving masses in mm (sorted by fosets)
		mm = sorted_data(:,nc);
	end

	for i = 1:length(indx)-1
		m_final(i) = sum(mm(indx(i)+1:indx(i+1)));
		foc_final(i,1:nbr_col_foset) = sorted_data(indx(i)+1,1:nbr_col_foset);
		if length(ind_contint)		
			clim{i} = belief(ind_b).bp(ind_p).form(ind_f).contint(sorted_data(indx(i)+1,nc-1)).limits;	
		end
		% handling log_expression when present
		mlx = '';
		ml = 10000;	
		for j = 1: indx(i+1)-indx(i)   
			tx = [];
			tx = ddblank(lg_expr{ndx(indx(i)+j)});
			if length(tx) & length(tx) < ml 
				mlx = tx;
				ml = length(tx);
			end
		end
		lg_expr_final{i} = [' ',mlx,' '];
		% I always need at least one blank in front and behind to be sure log expr 
		% has its operators with blanks on both sides
	end
	%
	% END handling contint case
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% BEG storing all data 
	%	
	belief(ind_b).bp(ind_p).form(ind_f).values = m_final';
	belief(ind_b).bp(ind_p).form(ind_f).focal_sets = foc_final;
	belief(ind_b).bp(ind_p).form(ind_f).log_expr = []; % to clear it
	belief(ind_b).bp(ind_p).form(ind_f).log_expr = lg_expr_final;	
	if length(ind_contint)
		belief(ind_b).bp(ind_p).form(ind_f).contint = []; % to clear it
		for i = 1:length(clim)
			belief(ind_b).bp(ind_p).form(ind_f).contint(i).limits = clim{i};
		end
	end
	%
	% END storing all data 
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
end	

% % % % % % % printing result for control while programming
% % % % % % belief(ind_b).bp(ind_p).form(ind_f).values
% % % % % % belief(ind_b).bp(ind_p).form(ind_f).focal_sets
% % % % % % belief(ind_b).bp(ind_p).form(ind_f).log_expr
% % % % % % if length(ind_contint)
% % % % % % 	for i = 1:length(clim)
% % % % % % 		belief(ind_b).bp(ind_p).form(ind_f).contint(i).limits
% % % % % % 	end
% % % % % % end


