function display_usefull_object_data(option)
% displaying all objects is a user friendly way, and only what is usefull
% option = 1, displayed on Matlab Command Window
%        = 2, listed in file TBM_Objects_List located in User_Data:Application_Data:name_of_application.prt_obj 

%
%   Version : Dec 19,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief pathname2TBMLABvers
global Which_Courier  memory_file_name

if option == 1
	fid = 1;
else
	out = handling_files('save_useful_object');
	add_saved_file = [memory_file_name.path,memory_file_name.name];
	text = ['Your data saved on file ',memory_file_name.name,' located in ',memory_file_name.path];
	fprintf(1,'%s\n',text)
	fid = fopen(add_saved_file,'wt');
	fprintf(fid,'To get correct alignment, use font Which_Courier\n\n',[]);
end

fprintf(fid,'\nTBMLAB\n',[]);
fprintf(fid,'Content of all the TBM objects, except the masses and focal sets.\n\n',[]);

% frame
fprintf(fid,'FRAME OBJECTS\n\n',[]);
for i = 1:length(frame)
	if ~isempty(frame(i).number)
		fprintf(fid,'frame full name   : %s\n',frame(i).full_name);
		fprintf(fid,'frame short name  : %s\n',frame(i).short_name);
		fprintf(fid,'frame type        : %s\n',frame(i).type);
		fprintf(fid,'frame cardinality : %i\n',frame(i).cardinality);
		switch frame(i).type,
		case {'Integers','Continuous'}
			fprintf(fid,'frame from to     : %i  %i\n',[frame(i).min,frame(i).max]);
		otherwise
			for j = 1:frame(i).cardinality
				fprintf(fid,'frame elements    : %s\n',frame(i).elements(j,:));
			end
		end
	end
	fprintf(fid,'\n\n',[]);
end

fprintf(fid,'STRUCTURE OBJECTS\n\n',[]);
for i = 1:length(structure)
	if ~isempty(structure(i).number)
		fprintf(fid,'structure full name          : %s\n',structure(i).full_name);
		fprintf(fid,'structure short name         : %s\n',structure(i).short_name);
		fprintf(fid,'structure most refined frame : %s\n',frame(pin_manipulation('frame','num2ind',structure(i).most_refined)).full_name  );
		for j = 1:length(structure(i).sub_frames)
			fprintf(fid,'structure subframe names     : %s\n',frame(pin_manipulation('frame','num2ind',structure(i).sub_frames(1))).full_name );		
		end
	
		if size(structure(i).mapping,1) >= 1
			fprintf(fid,'structure mapping matrix  : %s\n',num2str(structure(i).mapping(1,:)));
			for j = 2:size(structure(i).mapping,1)
				fprintf(fid,'                             : %s\n',num2str(structure(i).mapping(j,:)));
			end
		end
		fprintf(fid,'\n\n',[]);
	end
end

fprintf(fid,'ATTRIBUTE OBJECTS\n\n',[]);
for i = 1:length(attribute)
	if ~isempty(attribute(i).number)
		fprintf(fid,'attribute full name       : %s\n',attribute(i).full_name);
		fprintf(fid,'attribute short name      : %s\n',attribute(i).short_name);
		fprintf(fid,'attribute structure       : %s\n',structure(pin_manipulation('structure','num2ind',attribute(i).structure)).full_name  );
		for j = 1:length(attribute(i).index)		
			fprintf(fid,'attribute index : name  : %s\n', attribute(i).index(j).name);		
			fprintf(fid,'attribute index : frame : %s\n', frame(pin_manipulation('frame','num2ind',attribute(i).index(j).frame)).full_name );
		end
		fprintf(fid,'\n\n',[]);
	end
end


fprintf(fid,'VARIABLE OBJECTS\n\n',[]);
for i = 1:length(variable)
	if ~isempty(variable(i).number)
		fprintf(fid,'variable full name   : %s\n',variable(i).full_name);
		fprintf(fid,'variable short name  : %s\n',variable(i).short_name);
		fprintf(fid,'variable attribute   : %s\n',attribute(pin_manipulation('attribute','num2ind',variable(i).attribute)).full_name  );
		fprintf(fid,'variable frame       : %s\n',frame(pin_manipulation('frame','num2ind',variable(i).frame)).full_name  );
		
		ind_att = pin_manipulation('attribute','num2ind',variable(i).attribute);	
		for j = 1:length(attribute(ind_att).index)		
			ind_fra = pin_manipulation('frame','num2ind',attribute(ind_att).index(j).frame);	
			fprintf(fid,'variable index value : %s\n', ...
			[attribute(ind_att).index(j).name,' = ', frame(ind_fra).elements(variable(i).index_value(j),:)]);			
		end
	end
	fprintf(fid,'\n\n',[]);
end


fprintf(fid,'BELIEF OBJECTS\n\n',[]);
for i = 1:length(belief)
	if ~isempty(belief(i).number)
		fprintf(fid,'belief full name                : %s\n',belief(i).full_name);
		fprintf(fid,'belief short name               : %s\n',belief(i).short_name);
		if ~isempty(belief(i).agent)
			fprintf(fid,'belief agent                    : %s\n',belief(i).agent);
		end	
		if ~isempty(belief(i).time)
			fprintf(fid,'belief time                     : %i\n',belief(i).time);
		end
		if ~isempty(belief(i).ec)
			fprintf(fid,'belief evidential corpus        : %s\n',belief(i).ec);
		end
	
		fprintf(fid,'belief domain variables         : %s\n',variable(pin_manipulation('variable','num2ind',belief(i).domain_variable(1))).full_name  );		
		for j = 2:length(belief(i).domain_variable)
			fprintf(fid,'                                : %s\n',variable(pin_manipulation('variable','num2ind',belief(i).domain_variable(j))).full_name  );
		end	
		if length(belief(i).conditioning_variable) >= 1
			fprintf(fid,'belief conditioning variables   : %s\n',variable(pin_manipulation('variable','num2ind',belief(i).conditioning_variable(1))).full_name  );		
			for j = 2:length(belief(i).conditioning_variable)
				fprintf(fid,'                                : %s\n',variable(pin_manipulation('variable','num2ind',belief(i).conditioning_variable(j))).full_name  );
			end	
		end	
		% ok only if no conditinal, for conditional display which is the element for bp(i), aih!!! XXXXX
		for j = 1:length(belief(i).bp)
			fprintf(fid,'for belief potential %i \n',[j]);			
			for k = 1:length(belief(i).bp(j).form)
				fprintf(fid,'belief potential exisiting type : %s\n',belief(i).bp(j).form(k).type);	
			end		
		end
		fprintf(fid,'\n\n',[]);
	end
end

if fid > 1
	fclose(fid);
end




