function belief2nodes(num_bel,hd_axes)
% the function that prepares the nodes for the plot of network

% num_bel = list of belief object number to plot when opening network
% other may be added graphically later

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%
global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute  pin_variable 
global pin_mapping pin_belief 
global node pin_node link pin_link


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%`
%
% getting and verifying ind_bel and ind_var 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%`
ind_bel = pin_manipulation('belief','num2ind',num_bel);

if length(ind_bel) ~= length(num_bel)
	% error return
	disp('the request for belief2nodes is wrong: some belief objects do not exist.');
	return
end	

nbr_bel = length(ind_bel);
% getting the variables used in those bel
var_list = [];
for i = 1:nbr_bel	
	var_list = [var_list belief(ind_bel(i)).domain_variable  belief(ind_bel(i)).conditioning_variable];
end
num_var = unique(var_list); 
ind_var = pin_manipulation('variable','num2ind',num_var);

if length(ind_var) ~= length(num_var)
	% error return
	disp('the request for belief2nodes is wrong: some variable objects do not exist.');
	return		
end

nbr_var = length(ind_var);
nbr_nodes = nbr_bel + nbr_var; % number of nodes to plot

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Finding a 'nice' spread
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

h_skip = 150; % average width  between two nodes = 150
v_skip =  75; % average height between two nodes =  75 
nc = fix(sqrt(nbr_nodes));  % nbr of columns
nr = fix(nbr_nodes/nc) + 1; % nbr of rows
% we start at upper left , so vertical goes by negative, horizontal increases
pos_axes = get(hd_axes,'position');
pos = [ (pos_axes(1)+h_skip/2)  (pos_axes(2)+pos_axes(4))];
pos_mem = pos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Initiating the pin tables for node and link
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

pin_node = pin_manipulation('node','all',[]);
pin_link = pin_manipulation('link','all',[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%`
%
% building all belief nodes
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%`

% getting their pin
new_pin = get_next_pin('node');
kn = new_pin(2)-1;
nbr = new_pin(3)-1;
pin_pos = new_pin(1)-1;

% for every belief object
for i = 1:length(ind_bel)
	kn = kn + 1; 
	nbr = nbr + 1;
	pin_pos = pin_pos + 1;
	if i == nc * fix((i-1)/nc) + 1
		pos = [ pos_mem(1) pos(2)-v_skip];
	else
		pos = pos + [h_skip  0];	
	end	
	expr = belief(ind_bel(i)).short_name;
	expr(findstr('_',expr)) = ' ';% I must avoid _ and it lmeans subscript in text function 
	
	node(kn).number = nbr;
	node(kn).label = expr;
	node(kn).children = [];			% the number of the node representing a var used in my domain
	node(kn).parents = [];			% the number of the node representing a var used in my condit
	% I cannot yet build the real node(kn).children and node(kn).parents as numbers var missing
	node(kn).center = pos;
	node(kn).nature = 'belief';
	node(kn).shape = 'long_hexagon';
	node(kn).obj_num = num_bel(i);		% the number of the belief I represent
	node(kn).link = [];
	pin_node = [pin_node ; [pin_pos kn nbr]];
	
end % for each belief 
to = kn;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building all variable nodes, the families of belief nodes and the links
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

new_pin = get_next_pin('link');
lk = new_pin(2)-1;
nlk = new_pin(3)-1;	
lk_pos = new_pin(1)-1;

% position in axes
pos = pos_mem;
pos(2) = pos(2) - v_skip/2;

% for every variable
for i = 1:length(ind_var)
	kn = kn+1; 
	nbr = nbr + 1;
	pin_pos = pin_pos + 1;
	
	
	if i == nc * fix((i-1)/nc) + 1
		pos = [ pos_mem(1) pos(2)-v_skip];
	else
		pos = pos + [h_skip  0];	
	end	

	% the children  of a var node are those bel node parents
	% the parents of a var node are those bel node children
	my_parents = [];
	my_children = [];	
	for j = 1:length(ind_bel)
		if ismember(num_var(i),[belief(ind_bel(j)).domain_variable] )
			j_node = find([node(1:to).obj_num] == num_bel(j));
			j_node = pin_manipulation('node','pos2num',j_node);
			my_parents = [my_parents j_node];			
		elseif ismember(num_var(i),[belief(ind_bel(j)).conditioning_variable] )
			j_node = find([node(1:to).obj_num] == num_bel(j));
			j_node = pin_manipulation('node','pos2num',j_node);
			my_children = [my_children j_node];
		end
	end
	expr = variable(ind_var(i)).short_name;
	expr(findstr('_',expr)) = ' ';% I must avoid _ and it lmeans subscript in text function 
	
	% creating the node for this variable
	node(kn).number = nbr;
	node(kn).label = expr; 			
	node(kn).children = my_children;	% number of (belief) nodes where condit refers to me 
	node(kn).parents = my_parents;		% number of (belief) nodes where domain refers to me 
	node(kn).center = pos;
	node(kn).center_initial = pos;
	node(kn).nature = 'variable';
	node(kn).shape = 'oval';
	node(kn).obj_num = num_var(i); 		% the number of the variable I represent
	node(kn).link = [];	
	pin_node = [pin_node ; [pin_pos kn nbr]];
	
	% building the links that end on this variable
	for k = 1:length(my_children)
		lk = lk + 1;
		nlk = nlk + 1;
		lk_pos = lk_pos + 1;				
		link(lk).number = nlk;
		link(lk).nodes = [nbr   my_children(k)];
		link(lk).tip = 1;
		pk = pin_manipulation('node','num2ind',my_children(k));
		node(pk).link = [node(pk).link nlk];
		node(kn).link = [node(kn).link nlk];
		node(pk).parents = [ node(pk).parents  nbr ];			
		pin_link = [pin_link ; [lk_pos lk nlk]];
	end

	if length(my_children) > 0
		arrow = 1;
	else
		arrow = 0;
	end	

	for k = 1:length(my_parents)
		lk = lk + 1;
		nlk = nlk + 1;
		lk_pos = lk_pos + 1;
		link(lk).number = nlk;
		link(lk).nodes = [my_parents(k) nbr];
		link(lk).tip = arrow;
		
		pk = pin_manipulation('node','num2ind',my_parents(k));	
		node(pk).link = [node(pk).link nlk];
		node(kn).link = [node(kn).link nlk];
		node(pk).children = [ node(pk).children  nbr ];		
		pin_link = [pin_link ; [lk_pos lk nlk]];
	end

end % for each variable

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% belief, variable and link nodes exist 
% For belief-variable nodes,I only know what are their centers (which surely could be better selected???XXXX)
% localization of their external frame done once they have been picto once (depend on their picto)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
