function exit_choice_max = new_node_for_belief(bel_ind_list)
% building one belief node for each bel_ind(i), droped at left with its variables linked
% bel_ind = the index of the belief objects that must be plotted

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%


global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute  pin_variable 
global pin_mapping pin_belief 
global root_file_name save_path file_name_begin
global node pin_node link pin_link  need_uiresume flags flagstr scaling_coef

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Note on families
%
% how works children parents data (where we store node numbers)
% for a belief node: 
%		children = list of node number that refer to a var in domain of bel
%		parents  = list of node number that refer to a var in condit of bel
% for a variable node: 
%		children = list of node number that refer to a bel where I belong to condit 
%		parents  = list of node number that refer to a bel where I belong to domain 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% a nice blue color for a selected node
sel_col = [0.5 1 1];

% position are the axes
pos_axes = get(gca,'position');
% position for dropping the belief node
pos = [50 .9*pos_axes(4)];

for i_bel_ind = 1:length(bel_ind_list)
	bel_ind = bel_ind_list(i_bel_ind);
	pos(2) = pos(2) - 10; % so they don't overwrite each others
	
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% verifying this belief has not alreday been put into a node
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	if length(node)
		node_ind = find(strcmp({node(:).nature},'belief') & ...
		ismember([node(:).obj_num],pin_manipulation('belief','ind2num',bel_ind)));
		if length(node_ind)			 
			new_node(i_bel_ind) = 0; % it exist already, I skip
		else
			new_node(i_bel_ind) = 1; 
		end
	end

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	%
	% building one belief node
	%
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	
	if new_node(i_bel_ind)
		% getting next node pin for the belief node
		new_pin = get_next_pin('node');
		kn_bel = new_pin(2);  		% ind of node with belief
		nbr_bel_node = new_pin(3);  	% nbr of node with belief
		pin_pos_bel = new_pin(1);	% pos of the node in pin_node
		
		expr = belief(bel_ind).short_name;
		expr(findstr('_',expr)) = ' ';% I must avoid _ as it lmeans subscript in text function (as in Latex) 
		
		node(kn_bel).number = nbr_bel_node;
		node(kn_bel).label = expr;
		node(kn_bel).children = [];	% the number of the var node representing a var used in my domain
		node(kn_bel).parents  = [];	% the number of the var node representing a var used in my sondit
		node(kn_bel).center = pos;
		node(kn_bel).center_initial = pos;
		node(kn_bel).nature = 'belief';
		node(kn_bel).shape = 'long_hexagon';
		node(kn_bel).obj_num = belief(bel_ind).number;	% the number of the belief I represent
		node(kn_bel).link = [];
		% node(kn_bel).localization					created later
		% node(kn_bel).localization_init_sc			created later
		% node(kn_bel).hd_menu						created later	
		% node(kn_bel).hd_menu_opt					created later
		
		% updating pin_node
		pin_node = [pin_node ; [pin_pos_bel kn_bel nbr_bel_node]];
		
		% draw the belief node (and create the last 4 objects in node(kn_bel))
		picto_with_text_and_menu(kn_bel,'k',sel_col); 
		
		% arrow only if there are conditionig variables 
		% and only between bel node and its domain variable, see later
		if isempty(belief(bel_ind).conditioning_variable)
			arrow = 0;
		else
			arrow = 1;
		end
	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		%
		% building or finding the variable node
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		% changing the position for the created variable nodes
		pos(1) = pos(1) + 50; % just an horizontal shift
		
		% finding all variables in the belief object
		list_var_num = [belief(bel_ind).domain_variable   belief(bel_ind).conditioning_variable];
		
		if length(list_var_num) == 0
			STUPID % how comes, what is a belief without domain
		else
	
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			%
			% for every variable in the belief object:
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			kn = kn_bel;
			nbr = nbr_bel_node;
			pin_pos = pin_pos_bel;
			
			for i = 1:length(list_var_num)	% for every var
				
				if i <= length(belief(bel_ind).domain_variable)
					child = 1; % this variable is a domain var of bel
				else
					child = 0; % this variable is a condit var of bel
				end
			
				% node_ind = ind of the variable node (if exist) which represents var num = list_var_num(i)
				node_ind = find(strcmp({node(:).nature},'variable') & ...
				ismember([node(:).obj_num],list_var_num(i)));
				
				if length(node_ind) > 1
					STUPID2 % 2 nodes refer to the same variable, forbidden
				end
			
				if node_ind
					
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					%
					% variable already exists in the network, get its node index
					%
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					
					% storing its node nunber and indec for building links
					nbr_var_node = node(node_ind).number; 
					ind_var_node = node_ind;
					
					if child
						node(kn_bel).children = [node(kn_bel).children   node(node_ind).number];
						node(node_ind).parents = [node(node_ind).parents node(kn_bel).number];  
					else
						node(kn_bel).parents  = [node(kn_bel).parents  node(node_ind).number];  
						node(node_ind).children = [node(node_ind).children node(kn_bel).number];				
					end	 
					% build the links of belief towards these nodes and reverse
					
					% how works children parents
					% for a belief node: 
					%		children = list of node number that refer to a var in domain of bel
					%		parents  = list of node number that refer to a var in condit of bel
					% for a variable node: 
					%		children = list of node number that refer to a bel where I belong to condit 
					%		parents  = list of node number that refer to a bel where I belong to domain 
					
				else
			
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					%
					% var not yet in network, create a var node for it
					%
					%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
					
					kn = kn + 1;
					nbr = nbr + 1;
					pin_pos = pin_pos + 1;
					
					nbr_var_node = nbr; 
					ind_var_node = kn;
					
					var_ind = pin_manipulation('variable','num2ind',list_var_num(i));
					
					expr = variable(var_ind).short_name;
					expr(findstr('_',expr)) = ' ';
					
					pos(2) = pos(2) -50;
					
					node(kn).number = nbr;
					node(kn).label = expr; 			
					node(kn).children = []; % number of (belief) nodes where condit refers to me 
					node(kn).parents = [];  % number of (belief) nodes where domain refers to me 
					node(kn).center = pos;
					node(kn).center_initial = pos;
					node(kn).nature = 'variable';
					node(kn).shape = 'oval';
					node(kn).obj_num = variable(var_ind).number; % the number of the variable I represent
					node(kn).link = [];	
					
					% update pin_node
					pin_node = [pin_node ; [pin_pos kn nbr]];
					
					% draw the new variable node
					picto_with_text_and_menu(kn,'k',sel_col);
					
					if child
						node(kn).parents = nbr_bel_node;
						node(kn_bel).children = [node(kn_bel).children   node(kn).number];
					else
						node(kn).children  = nbr_bel_node;
						node(kn_bel).parents = [node(kn_bel).parents   node(kn).number];				
					end		
				
				end % the variable node is found or built
			
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				%
				% build the links from now belief to each variable
				%
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				
				new_pin = get_next_pin('link');
				lk = new_pin(2);
				nlk = new_pin(3);	
				lk_pos = new_pin(1);
				
				% update pin_link
				pin_link = [pin_link ; [lk_pos lk nlk]];
				
				link(lk).number = nlk;
				if child 
					link(lk).nodes =  [nbr_bel_node  nbr_var_node];
					link(lk).tip = arrow;
				else
					link(lk).nodes  = [nbr_var_node  nbr_bel_node];
					link(lk).tip = 0;
				end
			
				node(kn_bel).link = [node(kn_bel).link  nlk];
				node(nbr_var_node ).link = [node(ind_var_node ).link nlk];
				
				% plot the link between the belief node and the var node
				% a new var cannot belong to a previously built belief node
				% as it would have alrerady been built
				link(lk).hd_line = plot_line(lk,'k');
				
			end	% for each variable		
		end
	end
end	
exit_choice_max = max(new_node);
