function [hd_one_panel] = panel_tabulator(dim_table,pos_free,LLposition,heavy_lines,...
title_table,comment_table)

% option = 0 no upper no left data
% option = 1 = I want only an upper line
% option = 2 = I want only a left column
% option = 3 = I want both upper + left column

% hd_one_panel(i,j) i=1:nr,j=1:nc = cells in table
% hd_one_panel(nr+1,j) = upper line
% hd_one_panel(i,nc+1) = left column
% hd_one_panel(nr+1,nc+1) = upper_corner

% dim_table = nrow, ncol, option_borders, nbr_char_width, nbr_char_left_col
nr = dim_table(1);
nc = dim_table(2);
option = dim_table(3);
nbr_char_data = dim_table(4);
nbr_char_left = dim_table(5);

switch option
case 0
	lc = 0; ul = 0;
case 1
	lc = 0; ul = 1;
case 2
	lc = 1; ul = 0;
case 3
	lc = 1; ul = 1;
end	

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the dimensions of the boxes for the data (for nbr_char_data char)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hd_temp = uicontrol(gcf,...
'style','text','position',[2 2 500 5]); % any pos OK, just to run textwrap

fake_text(1:nbr_char_data) = 'w';
[outtext, pos_box] = textwrap(hd_temp,{fake_text});
pos_box(3) = ceil(pos_box(3));
pos_box(4) = ceil(pos_box(4));
% pos_box = size of the box with data
box_w = pos_box(3);
box_h = fix(1.3*pos_box(4));
sp_line = 1;
sp_col = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% the width of the left most column, in pos_left(3) (for nbr_char_left char)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

left_w = 0;
if nbr_char_left
	fake_text(1:nbr_char_left) = 'w';
	[outtext, pos_left] = textwrap(hd_temp,{fake_text});
	left_w = pos_left(3);
	delete(hd_temp) 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% horizontal lines they goes two to the left, one to the right, should be used with frames
% col 1 = the one to the left
% row 1 = the one up
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hori_start(1) = 0;
if lc
	hori_start(2) = left_w + sp_col;
else
	hori_start(2) = box_w + sp_col;
end
for i = 2:nc+lc
	hori_start(i+1) = hori_start(i) + box_w + sp_col;
end

vert_start(1) = 0;
vert_start(2) = box_h + sp_line;
for i = 2:nr+ul
	vert_start(i+1) = vert_start(i) + box_h + sp_line;
end
tot_ht = vert_start(nr+ul+1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% deciding where to put the table (if LLposition = [])
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if max(LLposition) == 0
	pos_fig = get(gcf,'position');
	LLposition(1) = (pos_fig(3) - hori_start(nc+lc+1))/2;
	LLposition(2) = (pos_fig(4) - tot_ht)/2;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% plotting the many boxes
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

nr1 = nr + ul;
pos = [ LLposition(1) LLposition(2)+nr1*(box_h+sp_line) 0 box_h ];

for i = 1:nr1
	pos(2) = pos(2) - box_h - sp_line;
	pos(1) = LLposition(1);
	
	if option == 1 | option == 3 % there is upper line
		if i == 1 			
			ii = nr + 1;
		else
			ii = i - 1;
		end	
	else % no upper line
		ii = i;
	end
	if option == 2 | option == 3 % left line
		pos(3) = left_w;
		hd_one_panel(ii,nc+1) = uicontrol(gcf,...
		'position',pos,'style','edit');	
		pos(1) = pos(1) + pos(3) + sp_col;
	end
	pos(3) = box_w;
	for j = 1:nc
		hd_one_panel(ii,j) = uicontrol(gcf,...
		'position',pos,'style','edit');
		pos(1) = pos(1) + pos(3) + sp_col;
	end
end			
% NB: edit comes from with a frame whereas text does not.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% plotting the heavy lines:  
% heavy_lines.vert{j} = [vert_num from to]
% heavy_lines.hori{j} = [hori_num from to]
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% drawing the external frame, always there
ii = 0;

% left
pos_hl = [LLposition(1)-2  LLposition(2)-2 3 vert_start(nr+ul+1)+1];
ii=ii+1;hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
'backgroundcolor','k');

% right
pos_hl = [LLposition(1)+hori_start(nc+lc+1)-2  LLposition(2)-2 3 vert_start(nr+ul+1)+1];
ii=ii+1;hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
'backgroundcolor','k');

% bottom
pos_hl = [LLposition(1)-2  LLposition(2)-2  hori_start(nc+lc+1)+1 3];
ii=ii+1;hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
'backgroundcolor','k');

% up
pos_hl = [LLposition(1)-2  LLposition(2)+tot_ht-2 hori_start(nc+lc+1)+3 3];
ii=ii+1;hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
'backgroundcolor','k');

total_width = hori_start(nc+lc+1)+3; % for the title and comments boxes

% the horizontal lines 
for ik = 1:length(heavy_lines.hori)
	d = heavy_lines.hori{ik};
	i_line = d(1);
	if length(d) ~= 1
		from = d(2); to = d(3);
	else
		from = 1   ; to = nc + lc + 1;
	end
	pos_hl = [ LLposition(1)+hori_start(from) ...
	LLposition(2)+tot_ht-vert_start(i_line)-2 ...
	hori_start(to)-hori_start(from)-1  sp_line+2 ];
	
	ii = ii + 1;
	hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
	'backgroundcolor','k');	
end

% the vertical lines
for ik = 1:length(heavy_lines.vert)
	d = heavy_lines.vert{ik};
	i_line = d(1);
	if length(d) ~= 1
		from = d(2); to = d(3);
	else
		from = 1   ; to = nr + ul + 1;
	end
	pos_hl = [ LLposition(1)+hori_start(i_line)-2 ...
	LLposition(2)+tot_ht-vert_start(to) ...
	sp_col+2  vert_start(to)-vert_start(from)-2];	
	ii = ii + 1;
	hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
	'backgroundcolor','k');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The table title
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isempty(title_table)
	pos_hl = [ LLposition(1)-2 ...
	LLposition(2)+tot_ht+2   total_width   15];	
	ii = ii + 1;
	hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
	'string',title_table,'horizontalalignment','center');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% The table comment
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if ~isempty(comment_table)
	[nwt posct] = textwrap(hd_hl(ii),{comment_table});
	pos_hl = [ LLposition(1)-2 ...
	LLposition(2)-(posct(4)+20-2)   total_width   posct(4)+10];
	ii = ii + 1;
	hd_hl(ii) = uicontrol('style','text','position',pos_hl,...
	'string',nwt,'horizontalalignment','left');
end

