function fmt_explaining
% a display of the matrices that underly the FMT
% no normalization anywhere

%
%   Version : May 1, 2003.
%   Copyright (c) 2003 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global gui_position  figure_data  flags  help_level help_label
global Which_Courier

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% building the various matrices XfrY, some do not exist
% forms = m b q bel pl , never normalized
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

card = 3;
nr = pow2(card);

for i = 1:nr
	J(i,nr+1-i) = 1;
end
one = ones(nr,1);
label = {'M','B','Q','Bel','Pl'} ; 

mat1 = eye(nr);mat1(1,2) = 1;mat1(3,4) = 1;mat1(5,6) = 1;mat1(7,8) = 1;mat1 = mat1';
mat2 = eye(nr);mat2(1,3) = 1;mat2(2,4) = 1;mat2(5,7) = 1;mat2(6,8) = 1;mat2 = mat2';
mat3 = eye(nr);mat3(1,5) = 1;mat3(2,6) = 1;mat3(3,7) = 1;mat3(4,8) = 1;mat3 = mat3';
mat123 = mat3*mat2*mat1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% opening the figure in tabulator
% one option = 6 basic table (M B Q)
% the other 8 with Bel,Pl
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

title = 'Explaining the Fast Mobius Transform (FMT). The b_from_m case';
text_comment = {['The FMT is to belief functions what teh FFT (Fast Fourier Transform) is to spectral analysis,',...
' a very efficient algorithm to compute the various potential forms.',...
' We show how it works for the b_from_m case.',...
' The upper set shows at left the b_frm_m matrix, and its decomposition that underlies the FMT.',...
' The bottom tabulator shows an example of FMT. Going through the columns from right to left, we have succcessively:',...
' the initail bba m, the impact of m1, the vector m1*m, the impact of m2, the vector m2*m1*m,',...
' the impact of m3, the vector b = m3*m2*m1*m.']};

dim_table = [8 8 3 4 4 ; 8 8 0 3 3 ; 8 8 0 3 3 ; 8 8 0 3 3; 8 7 3 10 5];

switch computer
case 'MAC2'
	h1 = 250; h2=20;   l1 = 10; ltab = 250; hskip1 = 290; hspk2 = 200;
case 'PCWIN'
	h1 = 260; h2=20;   l1 = 10; ltab = 250; hskip1 = 310; hspk2 = 220;
otherwise
	h1 = 260; h2=20;   l1 = 10; ltab = 250; hskip1 = 310; hspk2 = 220;
end

% % itab = 220; skip = 70;
LLposition(1,1:2) = [l1 h1];
LLposition(2,1:2) = LLposition(1,1:2) + [hskip1 0];
LLposition(3,1:2) = LLposition(2,1:2) + [hspk2 0];
LLposition(4,1:2) = LLposition(3,1:2) + [hspk2 0]; 
LLposition(5,1:2) =[300 h2];

heavy_lines(1).hori{1} = [2];
heavy_lines(1).vert{1} = [2];
for i = 2:4
	heavy_lines(i).hori = [];
	heavy_lines(i).vert = [];
end
heavy_lines(5).hori{1} = [2];
heavy_lines(5).vert{1} = [2];



title_table = {'b_from_m = m3*m2*m1','m3','m2','m1',''};
comment_table = {'','','','',''};

% dim_table = for each table = nrow, ncol, option_borders, nbr_char_width, nbr_char_left_col

[hd_one_panel,hd_title,hd_text] = tabulator(title,text_comment,dim_table,...
LLposition,heavy_lines,title_table,comment_table);

line_label = {'{}','a','b','ab','c','ac','bc','abc'};
uptx_label = {'b=m3*v3','m3','v3=m2*v2','m2','v2=m1*v1','m1','v1=m'};

for j = 1:8
	set(hd_one_panel{1}(j,9),'string',line_label{j},'horizontalalignment','center');
	set(hd_one_panel{1}(9,j),'string',line_label{j},'horizontalalignment','center');
	set(hd_one_panel{5}(j,8),'string',line_label{j},'horizontalalignment','center');
end	
for j = 1:7
	set(hd_one_panel{5}(9,j),'string',uptx_label{j},'horizontalalignment','center');
end	

for i_tab = 1:4
	switch i_tab
	case 1		
		mat = mat123;
	case 2		
		mat = mat3;
	case 3		
		mat = mat2;
	case 4		
		mat = mat1;
	end
	for i = 1:8
		for j = 1:8
			if mat(i,j) == 0
				tx = ' ';
			else
				tx = sprintf('%1i',mat(i,j));
			end
			set(hd_one_panel{i_tab}(i,j),'string',tx,'horizontalalignment','center');
		end
	end
end

v1 = fix(100*sort([rand(nr-1,1);1]))/100;
v1 = [v1(1); diff(v1)];
v2 = mat1*v1;
v3 = mat2*v2;
v4 = mat3*v3;


for j = 1:7
	switch j
	case 1
		v = v1;
	case 2		
		v = {'1','1+2','3','3+4','5','5+6','7','7+8'};
	case 3		
		v = v2;
	case 4		
		v = {'1','2','1+3','2+4','5','6','5+7','6+8'};
	case 5		
		v = v3;
	case 6		
		v = {'1','2','3','4','1+5','2+6','3+7','4+8'};
	case 7
		v = v4;
	end	
	for i = 1:8	
		if ismember(j,[1 3 5 7])
			if v(i) == 0
				tx = ' ';
			else
				tx = sprintf('%5.3f',v(i));
			end
		else
			tx = v(i);
		end
		set(hd_one_panel{5}(i,8-j),'string',tx,'horizontalalignment','center');		
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% put its menus
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ind_fig = length(figure_data) + 1;
k=0;
k=k+1;make_fig_menu(ind_fig,k,'File',0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Quit',1,'general_clb(2)','on'); 
k=k+1;make_fig_menu(ind_fig,k, help_label,0,'','on');
k=k+1;make_fig_menu(ind_fig,k,'Help', 3,'general_clb(1)','on');

menu_manager('create',ind_fig)

flags = 0;
go_on = 1;
while go_on
	uiwait(gcf)	
	if flags == 1	
		gcf_mem = gcf;
		help_level_mem = help_level;
		help_level = 1;
		help_text_handles = display_helps(1,49,Which_Courier);
		% text 49 in init_stored_texts 
		delete(help_text_handles)
		help_level = help_level_mem;
		figure(gcf_mem);
	else
		% set(gcf,'DefaultuicontrolUnits','normalized')
		% choice = text_with_n_buttons(gcf,'To quit, select it',{'Quit'}, [-1]);
		delete(gcf)
		go_on = 0; % we quit
		figure_data(ind_fig) = [];
	end	
end

