function handle_list = listing_existing(option, position_in, Wdw_Cmd_handle,param)
% listing in a ListBox located at choosen position
% the objects (the name is in option) that already exist.

% option 	- the kind of object to list. ex.: 'frame', 'structure', ....
% position	- the position of the list (4 numbers)
% Wdw_Cmd_handle	- the handle of the figure where list is displayed
% param				- extra parameters needed fot the potentials

% By default, it is a single selection, 
% if you want multiple selection, in the calling function put:
% set(handle_list,'Max',1000,'Value',[])
% handle_list(1) = title text
% handle_list(2) = listed text
% handle_list(3) = title frame 
% handle_list(4) = external frame  

% whenever relevant rebuild pin_object to be sure it is correct

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

global frame structure mapping attribute variable belief
global pin_frame pin_structure pin_attribute pin_variable 
global pin_mapping pin_belief
global nice_yellow

skip_me = 0;
list = {}; % we have a list empty when there is nothing to list
position = position_in;

switch option
case 'frame'
	% listing exisitng frames .full_name and .number	
	object = frame;
	useless = pin_manipulation('frame','all',[]);	
	pin_list = pin_frame;
case 'structure'
	% listing exisitng structure.number	
	object = structure;
	useless = pin_manipulation('structure','all',[]);	
	pin_list = pin_structure;	
case 'mapping'
	object = mapping;
	useless = pin_manipulation('mapping','all',[]);	
	pin_list = pin_mapping;
	skip_me = 1;	% we go direct to print
	for i = 1:length(object)
		iobj = pin_mapping(i,2);
		list{i} = strcat('C',num2str(object(iobj).number),' from F',...
		num2str(object(iobj).finer),' to F',num2str(object(iobj).coarser));
	end
case 'attribute'
	object = attribute;
	useless = pin_manipulation('attribute','all',[]);	
	pin_list = pin_attribute;	
case 'variable'
	object = variable;
	useless = pin_manipulation('variable','all',[]);	
	pin_list = pin_variable;	
case 'belief'
	object = belief;
	useless = pin_manipulation('belief','all',[]);	
	pin_list = pin_belief;
case 'potential'
	object = belief(param(1)).bp;
	pin_list = repmat([1:length(object)]',1,3);	
case 'form'
	object = belief(param(1)).bp(param(2)).form;
	pin_list = repmat([1:length(object)]',1,3);	
	
	
	
case 'all_potentials'
	% building a list with ALL potentials
	% long name like B# bp# m BetP q
	object = []; % so I don't get a warning on uninitialize 
	ij = 0;
	for ib = [pin_belief(:,2)]'
		for j = 1:length(belief(ib).bp(:))
			ij = ij + 1;
			list{ij} = ['B',num2str(belief(ib).number),' bp',num2str(j)];
			for k = 1:length(belief(ib).bp(j).form)
				list{ij} = [list{ij},' ',belief(ib).bp(j).form(k).type];
			end
		end
	end
	skip_me = 1;	% we go direct to print
case 'frame_elements'
	list = param;
	skip_me = 1;	% we go direct to print
case 'menu'
	list = param;
	skip_me = 1;	% we go direct to print
	position(3) = 1.5*position(3); % I need more room for that list
case 'general'
	list = param;
	skip_me = 1;	% we go direct to print
otherwise
	disp('error in request to listing existing')
	return
end

% creating labels for object name in listbox (its number and full_name)
if skip_me == 0 % no labels requested, we just display values in list
	if length(object) 
		% it exists something
		kk=0;		
		for jj = 1:size(pin_list,1)
			i = pin_list(jj,2); % the index
			kk = kk + 1;
			switch option
			case 'potential'
				list{kk} = ['bp ', num2str(i),' ',object(i).condition,' '];
				for jkl = 1:length(object(i).form)
					list{kk} = [list{kk},' ',object(i).form(jkl).type];
				end
				
			case 'form'
				list{kk} = ['bp_form ', num2str(i),' ', object(i).type,' ',object(i).representation];
			otherwise
				list{kk} = [num2str(object(i).number),' ',object(i).full_name] ;
			end	
		end	
	else
		list = {''};
	end
	% displaying title and list
	% by default you can select ONE of them (for looking at its content, and maybe modifying it)
end

% hd_fake = uicontrol(gcf,'position',[1 1 800 600],'style','text');
hd_fake = uicontrol(gcf,'position',[.001 .001 .99 .99],'style','text');
[useless, size_text] = textwrap(hd_fake,list);
delete(hd_fake);

% initially position is the top bottom space
% I raise bottom and schrink heigh
% I schrink width for aesthetical reasons
scroll_ht = .025; % for the horizontal scroll bar
position(4) = min(position(4),(1.1*size_text(4)+scroll_ht));
position(2) = .99 - .053 - position(4);
position(3) = position(3) - .01;

% posit1	the external frame for the title
% posit2	the list itself
% posit0 	the external overall frame 
% ppos   	the title text
posit1 = [ position(1)   (position(2)+position(4)+.01)    position(3)   .04];
posit2 = [ position(1)    position(2)      position(3)     position(4)];
posit0 = [ position(1)-.006   posit2(2)-.006   position(3)+.012  posit2(4)+posit1(4)+.04];

% overall frame 
handle_list(4) = uicontrol('Parent', Wdw_Cmd_handle,...
'position',posit0,...
'backgroundcolor',[.7 .7 .7],...
'Style','frame');

switch computer
case 'MAC2'
	ppos = posit1 + [.005 .005 -.01 -.01];
case 'PCWIN'
	ppos = posit1 + [.005 .01 -.01 -.015];
otherwise
	ppos = posit1 + [.005 .01 -.01 -.015];
end

% title frame
handle_list(3) = uicontrol('Parent', Wdw_Cmd_handle,...
'position',posit1,...
'backgroundcolor',nice_yellow,...
'Style','frame');

% title text
handle_list(1) = uicontrol('Parent', Wdw_Cmd_handle,...
'position',ppos,...
'string',upper(option),...
'Style','text',...
'backgroundcolor',nice_yellow,...
'foregroundcolor','r');

% list box with list of exisiting objects
handle_list(2) = uicontrol('Parent', Wdw_Cmd_handle,...
'style','listbox',...
'position',posit2,...
'string',list,...
'min',0,...
'max',1000,...
'value',[], ...
'Callback', 'uiresume;',...
'Enable','off'); % 
% by default 1000 selectable lines, none preselected.
% ButtonDownFcn seems not handled by ListBox 
% set(handle_list(2),'ButtonDownFcn','callback_key_hit(1);uiresume;');
% 'Callback', 'callback_key_hit;uiresume;',...
