function [answer,choice] = dialog_queries(Wdw_Cmd_handle,title,queries_in,...
default_answer,positionLL,MenuData, nbr_buttons,...
hidden_querries, tooltip, index_set_but, pos_help_text)

global CallBackData CallBackCell
global button_size   keyhit
% uicontrol to get answers to a list of queries

% input 
% Wdw_Cmd_handle= handle of the FIGURE where queries window is to be displayed
% title			= a title that show up at top, can be empty {}
% queries_in 	= the list of queries (see below)
%				  if last character is #, answer MUST be a number
% default_answer= text put in the answer boxes at onset, see below, can also be {}
% positionLL 	= lower left position of window in normalized units, 2 values
%				= [] centered 
%				= [0 0] upper left
% nbr_buttons	= 1 only OK, 2 buttons OK and Cancel, 3 id + Help
%				  option 3 not fully ready, option 1 transformed into option 2.
% hidden_querries = the querries that are not shown and not active
%   			= []    or    [i j k] 
% tooltip		= short help text attached to the querry texts; show up with small delay
%				  string 'text', if [] no tooltip to show
% index_set_but	= a nunber that tells which set of long help texts is to be used
%				  to see the displayed texts, see get_help_text_for_set_buttons
%				  0 = no long help texts
% pos_help_text = UR or UL or Other (decided by call)

% when pushing button OK, it deletes all ui_objects generated here,

% output
% answer = the list of answers given to each query {{...},{...}}
% choice : 0 = initial_value, 1 = OK, 2 = Cancel, 3 = Help

% MenuData = olbject with all Menu requests.
% i = each menu, there can be several
% MenuData(i).number = number of the query for which it applies (from 1 to number_queries)
% MenuData(i).label{j} = 'text': the label to be shown
% MenuData(i).clb{j}   = 'text': the callback function 
%  		like 'fct 4' (4 is option '4') or 'fct(4)' in which case option is a number
% 		option not very sophisticated must be a number here

% avoid WindowButtonMotionFcn and a dialog_queries to be open simultaneiously

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% The queries in queries_in can be written as 
% if only one
% 		queries = 't',  NO FORBIDDEN
%		queries = {'t'}, 
%		queries = {{'t'}},
% if several
% 		queries = {'t','r'},
%		queries{1} = 't'; queries{2} = 'r';
% in fact the {} are first eliminated up to the string level,
% then restore as required by wraptext
% what is not ok is queries = {{'t','r'}}, as a.o. length not OK (1 and not 2)
% default_answer =  same as queries
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% an horror in uicnotrol
% if you put default = {' ',' '}, answer = {'a','b'}
% if you put default = {'',''},   answer = {{'a'},{'b'}}
% as far as I want answer = {'a','b'}, it must be fixed, as done herein

% another horror: when you write ans = dialog_queries.... ans is NOT ok,
% due to confusion with another ans is the system!!! 
% never use ans

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% preparing for long helps
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

% a set of global used only for callback, clear when button clicked
global  gui_position   width_long_text_help_window
global  button_num_where_pointer  long_text_help   position_figure_for_text  
global  pos_button_in_pixels    hd_lg_tx_fig  top_position_for_text   help_level

% deciding where to display help text

if isempty(pos_help_text)
	pos_help_text = 'UL'; % just to avoid crash in switch, but normaly unused
end

switch pos_help_text
case 'UL' 
	position_figure_for_text = [2*gui_position(1,1) 1.1*gui_position(2,1) .9*gui_position(3:4,1)'];
case 'UR'
	position_figure_for_text = [1.1*gui_position(1:2,5)' ...
	width_long_text_help_window*gui_position(3,5)   .9*gui_position(4,5)];
case 'Other'
	% I don't know where is the image, and I put help above comments figure
	position_figure_for_text = gui_position(:,8)';		
otherwise
	% nothing 
end

button_num_where_pointer = 0;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
%
% preparing for querries
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	

if exist('answer') 
	clear answer; 
end % to avoid old garbage

if ischar(queries_in) % to protect against queries_in = 'ttt'
	queries_in = {queries_in};
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% list of queries with a menu
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

queries = queries_in;
nbr_queries = length(queries); % nbr of lines with queries
menu_qu = zeros(nbr_queries,1);
if exist('MenuData')
	if ~isempty(MenuData)
		for i = 1:length(MenuData)
			ind = MenuData(i).number;
			menu_qu(ind) = i ; % so I know which data i for which query
			% 			% adding '(MENU)' if menu query
			% 			queries{ind} = [queries_in{ind},' (MENU).'];
		end
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% list of queries that require a NUMERIC answer (those ending with #)
% if not a number, system cycle up to when the number will be there
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

list_num_q = []; % the list with the index of those queries which answer nmust be a number

for iq = 1:nbr_queries
	if strcmp(queries{iq}(length(queries{iq})),'#')
		list_num_q = [list_num_q iq];
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% preparing sizes
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

posFig = get(Wdw_Cmd_handle,'Position') ; % not normalized as it is a figure
% needed to normalize fixed height and width

% size of button at bottom
button_h = min(button_size(2)/posFig(4),.99);	% normalized
button_w = min(button_size(1)/posFig(3),.99);	% normalized

% sizes of query_box
height_query_frame =  min(20/posFig(4), .99);	% normalized
width_query_frame  = min(190/posFig(3), .98); % normalized but less than half the width of the figure
% spacing between boxes 
vspace_inter_boxes = .015;	% normalized

% width for the whole window
total_width = width_query_frame + .01;% .005 on both sides

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% computing height 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% establishing heigth of each query after wrapping them 
% output are vectors, same length as queries
[wrapped_text, v_space] = fit_text_in_box(Wdw_Cmd_handle, queries, width_query_frame);
v_space_for_qu = v_space + vspace_inter_boxes + height_query_frame;

% establishing heigth of title after wrapping it
if not(isempty(title)) 
	[wrapped_title, title_space] = fit_text_in_box(Wdw_Cmd_handle, title, width_query_frame);
else
	title_space = 0;
	wrapped_title = [];
end

% space available for queries
free_v_space = 1 - title_space - .01 - button_h ; 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% computing number of query_boxes (for the case of many queries)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

cumulated_h = 0;
nbr_qu_boxes = 0;
for iq = 1:nbr_queries
	cumulated_h = cumulated_h + v_space_for_qu(iq);
	if cumulated_h > free_v_space
		nbr_qu_boxes = nbr_qu_boxes + 1;
		up_limit(nbr_qu_boxes) = iq - 1;
		cumulated_h = v_space_for_qu(iq);
	end
end
% handling the last query_box 
nbr_qu_boxes = nbr_qu_boxes + 1;
up_limit(nbr_qu_boxes) = nbr_queries;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
% handling each query box
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% % % % % a callback on the figure so that if I hit return and was on a editatble text I erase text
% % % % clb_sel_text = ['disp(''passe1'');if strcmp(get(gcf,''CurrentCharacter''),char(65)) & strcmp(get(gco,''usedrdata''),''correctionOK'');',...
% % % % 'set(gco,''string'','''');disp(''passe2'')end'];				
% % % % set(Wdw_Cmd_handle,'KeyPressFcn',clb_sel_text)

to = 0;
go_on = 1;
qu_box = 0;

while go_on 
	qu_box = qu_box + 1;
	if qu_box > nbr_qu_boxes
		go_on = 0;
	else
		from = to + 1; % queries from ...  to ...
		to = up_limit(qu_box);	
		
		total_height = sum(v_space(from:to)) + .01 + button_h + ...
		(height_query_frame + vspace_inter_boxes)*(to-from+1);
		
		if not(isempty(title)) 
			total_height = total_height + title_space + .01;
		end
	
		% getting the position of the frame
		if isempty(positionLL) % empty = center
			position = [ .5-total_width/2   .5-total_height/2 ...
			total_width  total_height ];
		elseif positionLL == [0 0] % just 1 = upper left
			position = [.01   .99-total_height   total_width    total_height]; 
		else
			position = [ positionLL(1)   positionLL(2) ...
			total_width     total_height ];			
		end
	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% 
		% draw the external frame
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		ikk = 1; 
		handles_clq(ikk) = uicontrol(...
		'Parent',Wdw_Cmd_handle, ...
		'Position',position, ...
		'BackgroundColor',[.8 .8 .8],...
		'Style','frame');				
		
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% 
		% print title in RED if not []
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		if not(isempty(title)) 	
			pos = [	position(1) + .005 ... 
			position(2) + total_height - title_space - vspace_inter_boxes ...
			width_query_frame   title_space + .003];
			
			posbox = [ position(1)  pos(2)-.002  position(3) pos(4)+.012];
			ikk = ikk+1;
			handles_clq(ikk) = uicontrol(...
			'Parent',Wdw_Cmd_handle, ...
			'Position',posbox, ...
			'Style','frame');
			ikk = ikk+1;
			
			handles_clq(ikk) = uicontrol(...
			'Parent',Wdw_Cmd_handle, ...
			'Position',pos, ...
			'Style','text', ...
			'ForegroundColor',[1 0 0],...
			'String',wrapped_title{1});
			total_height = total_height - title_space - .01;
		end
	
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		% 
		% print queries and prepare answer boxes
		%
		%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
		
		pos = [	position(1) + .005 ...
		position(2) + total_height  ...
		width_query_frame      height_query_frame ];
		
		% the queries themselves
		ok = 0; % ok = 1 if all required numerical answers in list_num_q are numbers
		% when answer is not a number str2num returns []	
		ikm = ikk; % memorizing the handle position 
		mpos = pos;
		pos_text_querry = [];
		ijq = 0;
		while ~ok
			
			for jq = from:to 
				
				% one querry 
				pos(2) = pos(2) - v_space(jq) - vspace_inter_boxes ;
				pos(4) = v_space(jq) ;
				ikk=ikk+1;
				handles_clq(ikk) = uicontrol(...
				'Parent',Wdw_Cmd_handle, ...
				'Position',pos, ...
				'Style','text', ...
				'HorizontalAlignment','left',...
				'BackgroundColor',[.8 .8 .8],...
				'String',wrapped_text{jq});	
				
				% the answers to the querry
				pos(2) =   pos(2) - height_query_frame;
				pos(4) = 	height_query_frame;
				
				% those queries displayed in the present frame, there may be several frames
				if isempty(default_answer)
					def_ans = {''}; % I put a blank to avoid accident if empty
				else
					if isempty(default_answer{jq})
						def_ans = {''}; 
					else		
						def_ans = default_answer{jq};
					end		
				end			
				ikk=ikk+1;
				input_handle(jq) = ikk;
				% making sure format of text is OK
				temp_text = def_ans;
				while iscell(temp_text) % it should be a string
					temp_text  = temp_text{1}; % going from cell toward string
				end
			
				% depending if there is a precooked answer in a pop up menu
				if ~menu_qu(jq) 
					% case no pop up menu				
					handles_clq(ikk) = uicontrol('Parent',Wdw_Cmd_handle, ...
					'Position',pos , ...
					'Style','edit',...
					'String',def_ans,...
					'HorizontalAlignment','left',...
					'BackGroundColor',[1 1 1]);	
				else
					% case with pop up menu
					ijk = menu_qu(jq);
					% the precooked answers for the pop up menu
					str_popup = {};
					str_popup{1} = def_ans;
					for nq = 2:length(MenuData(ijk).label)+1
						str_popup{nq} = MenuData(ijk).label{nq-1};
					end
				
					handles_clq(ikk) = uicontrol('Parent',Wdw_Cmd_handle, ...
					'Position',pos , ...
					'Style','popupmenu',...
					'String',str_popup,...
					'HorizontalAlignment','left',...
					'BackGroundColor',[1 1 1]);
					
					CallBackData(ijk) = handles_clq(ikk);
					
					% 'String',def_ans,...
					% 					disp('RRRRRRRRRRRR')
					% 					handles_clq(ikk)
					%                   the precooked answers for the pop up menu					
					% 					hand_type_fig = uicontextmenu('Parent',Wdw_Cmd_handle);
					% 					set(handles_clq(ikk),'UIContextMenu',hand_type_fig)
					% 					for nq = 1:length(MenuData(ijk).label)
					%                         item_menu(nq) = uimenu(hand_type_fig, 'Label',...
					%                             MenuData(ijk).label{nq}, ...
					%                             'Callback', MenuData(ijk).clb{nq});
					% 					end 				
				end		
			
				% saving position of the text 	
				ijq = ijq + 1;
				pos_text_querry(1,ijq) = handles_clq(ikk); % the answer
				pos_text_querry(2,ijq) = handles_clq(ikk-1); % the querry
				
				% adding the tooltip if they exist
				if ~isempty(tooltip)
					set(handles_clq(ikk),'TooltipString',tooltip{jq});
				end
			end	 % for jq = from:to 
		
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
			%
			% display of long help texts
			% but neverf used so I ancel it as it clash with entering data 
			% because of WindowButtonMotionFcn
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			% 			if  help_level  == 2
			% 				if index_set_but % we have long help texts (else skip)
			% 					% getting the buttons positions in pixel and their help texts	
			% 					[long_text_help, pos_button_in_pixels] = get_help_text_for_set_buttons(Wdw_Cmd_handle, ...
			% 					pos_text_querry(1,:),[index_set_but from to]);
			% 					
			% 					% creating the long help text figure
			% 					hd_lg_tx_fig = figure('Position',position_figure_for_text,'UserData','TBM_Figure');
			% 					
			% 					top_position_for_text = .99;
			% 					if ~strcmp(pos_help_text,'Other')
			% 						% writing its title except if in comments window as already quite small
			% 						title_lg_tx = 'Button attached Help';
			% 						hd_title_lg_tx = uicontrol('Parent', hd_lg_tx_fig,...
			% 						'Style','Text',...
			% 						'Position', [.005 .945 .99 .05],...
			% 						'String',title_lg_tx,...
			% 						'ForegroundColor','blue',...
			% 						'HorizontalAlignment','center',...
			% 						'FontSize',14);
			% 						top_position_for_text = .93;
			% 					end
			% 					% ordering that pointer position must be followed.
			% 					% call abck is get_button_under_pointer
			% 					set(Wdw_Cmd_handle,'WindowButtonMotionFcn','get_text_for_button_pointer;')						
			% 				end
			% 			end		
			
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%	
			%
			% some querries are disable and invisible
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			if ~isempty(hidden_querries)
				for i = 1:length(hidden_querries)
					if hidden_querries(i) >= from &  hidden_querries(i) <= to						
						set(pos_text_querry(1,hidden_querries(i)-from+1),'visible','off','enable','off');
						set(pos_text_querry(2,hidden_querries(i)-from+1),'visible','off','enable','off');
					end
				end
			end
		
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			% 
			% put two buttons to say 'I am thru, read my answers' or Cancel 
			% OK is default thus RETURN, Cancel is also ESC
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			% the return from button by hitting a key
			keyhit = 0;
			set(Wdw_Cmd_handle,'KeyPressFcn','callback_key_hit;uiresume;')
			
			% nbr_buttons = 1 or 2 produce OK Cancel
			
			if	nbr_buttons == 1
				nbr_buttons = 2;
			end
		
			pos = [ 0    position(2) + .005     button_w     button_h ];
			if nbr_buttons <= 2
				pos1 = pos;
				pos2 = pos;
				pos1(1) = position(1) + total_width/2 - button_w -.01;
				pos2(1) = position(1) + total_width/2 + .01;
			else % maybe one day will be used for ???
				pos1 = pos;
				pos2 = pos;
				pos3 = pos;
				pos1(1) = position(1) + total_width/4 -.02 - button_w/2;
				pos2(1) = position(1) + total_width/2 - button_w/2 ;
				pos3(1) = position(1) + 3*total_width/4 +.02 - button_w/2;
			end
		
			ikk=ikk+1;				
			handles_clq(ikk) = uicontrol('Parent',Wdw_Cmd_handle, ...
			'Position',pos1, ...
			'String','OK',...
			'Style','PushButton',...
			'BackgroundColor',[1 0 0],...
			'Userdata','notOK',...
			'Callback','set(gco,''userdata'',''OK'');uiresume;');
			
			ikk=ikk+1;				
			handles_clq(ikk) = uicontrol('Parent',Wdw_Cmd_handle, ...
			'Position',pos2, ...
			'String','Cancel',...
			'Style','PushButton',...
			'BackgroundColor',[1 0.4 0.4],...
			'Userdata','notOK',...
			'Callback','set(gco,''userdata'',''Cancel'');uiresume,');
			
			if nbr_buttons == 3
				ikk=ikk+1;				
				handles_clq(ikk) = uicontrol('Parent',Wdw_Cmd_handle, ...
				'Position',pos3, ...
				'String','Help',...
				'Style','PushButton',...
				'BackgroundColor',[1 0.4 0.4],...
				'Userdata','notOK',...
				'Callback','set(gco,''userdata'',''Help'');uiresume,');
			end
		
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			% 
			% the loop up to I select OK of cancel			
			%
			%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
			
			choice = 0; 
			ok3 = ' ';
			
			figure(Wdw_Cmd_handle);
			
			% i make sure WindowButtonMotionFcn is off
			% I store its past value, balnk it, and will resinatll it later
			WindowButtonMotionFcn_mem = get(gcf,'WindowButtonMotionFcn');
			set(gcf,'WindowButtonMotionFcn','')
			
			while choice == 0
				
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				%
				uiwait(Wdw_Cmd_handle)
				%
				%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
				% one goes thru here only after pushing one of the 2 or 3 buttons, 
				% not after just filling a field in the text area
				% so you fill the fields as you want, and conclude with the buttons OK, cancel
				
				% seeing whih button was selected
				if nbr_buttons <= 2
					ok1 = get(handles_clq(ikk-1),'userdata');
					ok2 = get(handles_clq(ikk),'userdata');
				else
					ok1 = get(handles_clq(ikk-2),'userdata');
					ok2 = get(handles_clq(ikk-1),'userdata');
					ok3 = get(handles_clq(ikk),'userdata');
				end	
			
				switch keyhit
				case 1
					ok1 = 'OK';
				case 2
					ok2 = 'Cancel';
				end		
			
				if strcmp(ok1,'OK')
					choice = 1;
				elseif strcmp(ok2,'Cancel')		
					choice = 2;
				elseif  strcmp(ok3,'Help')	
					choice = 3;
				end
			end
		
			% restore WindowButtonMotionFcn
			set(gcf,'WindowButtonMotionFcn',WindowButtonMotionFcn_mem);
			
			% get answers, case choice = 3 not handled as not used
			ok = 1;
			if choice == 2 % we ask for cancel
				answer = default_answer;
				go_on = 0;
			else
				for jq = from:to
					val_sel = get(handles_clq(input_handle(jq)),'Value');
					answer_temp = get(handles_clq(input_handle(jq)),'String');
					if val_sel == 0
						answer{jq} = answer_temp;
					else
						answer{jq} = answer_temp{val_sel};
					end
				
					% adaptatioon to get answer{jq} = 'ttt' and not {'ttt'}
					while ~ischar(answer{jq}); answer{jq} = answer{jq}{1}; end
					
					if ismember(jq,list_num_q)
						if isempty(str2num(answer{jq}))
							ok = 0;
						end
					end
				end
				if ok == 0 % I want to keep what has alreday been answered
					hdw = warndlg('A number is required. Fill again.');
					uiwait(hdw);
					for iiq = from:to
						default_answer{iiq} = answer{iiq};
					end
					delete(handles_clq(ikm+1: ikk));
					ikk = ikm; % I restore the handle index where I was
					pos = mpos;
				end
			end
		
		end % for the while ~ok (number control)
		delete(handles_clq);
		clear handles_clq
		
	end % end of the several querry_boxes : for qu_box = 1:nbr_qu_boxes
end % while go_on

% if ishandle(hd_lg_tx_fig) ; delete(hd_title_lg_tx); delete(hd_lg_tx_fig); end

% clearing the global data without using clear global that might misbehave?
button_num_where_pointer = []; 
position_figure_for_text = [];
pos_button_in_pixels = [];
top_position_for_text = [];

% long_text_help = [];   
% hd_lg_tx_fig = [];

set(Wdw_Cmd_handle,'KeyPressFcn','')
