function answer = matrix_of_queries(nr, nc, label_row, label_col, ...
	def_val, position, option)
% drawing a (nr x nc) matrix where each cell can be  

% label_row are the labels (text) written at left as row value
% from top to bottom
% label_col are the labels (text) written at top as column value
% from left to right
% I need label{i} = 'text'

% input
% nr		: nbr of rows
% nc 		= nbr of columns
% label_row	: to be displayed for rows     label_row{i} = 'text'
% label_col	: to be displayed for columns  label_col{i} = 'text'
% def_val 	: option 1, unused, 2 numbers, 3 strings, put in cell as default
% position 	: overal position of object (4 digits) normalized
% option	: 
% 		= 1	: just click in which case contents goes from 0 to 1
% 		= 2	: filled with a number
% 		= 3	: filled with a text

% output 
% answer(i,j) for option 1 and 2, answer{i,j} for option 3
% the values you have in the table when you type OK

%
%   Version : Oct 25,02.
%   Copyright (c) 2002 by Philippe Smets, Brussels, Belgium.
% 	Author: Philippe Smets
%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% initiating 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch option
case 1
	style = 'togglebutton';
	title = {'Select cells, then click on me.'};

case 2
	style = 'Edit';
	for i=1:nr;for j=1:nc
		text(i,j) = num2str(def_val(i,j));
	end;end
	title = {'Fill numbers, then click on me.'};

case 3
	style = 'Edit';
	text = def_val;
	title = {'Fill texts, then click on me.'};

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% fixing the overal size, button should not be too large
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


button_h = .05; button_w = .25; % button size

schrink = .002; % little space for avoiding line atop each other

max_side = .08; % maximum for the buttons

% horiz_shift for row labels, vert_shift for column labels
horiz_shift = .05;  vert_shift = .05;

% I only keep LL corner of position

% where I display the matrix of buttons
position_array	= [position(1)+schrink    position(2)+button_h+2*schrink...
		position(3)-2*schrink   position(4)-button_h-3*schrink];
dim = [(position_array(3)-horiz_shift)/nc    (position_array(4)-vert_shift)/nr];
% dim are the button dimensions
dim =[  min(dim(1),max_side)   min(dim(2),max_side)]; % avoiding BIG buttons
position(3) = nc*dim(1)+horiz_shift+-2*schrink;
position(4) = nr*dim(2)+vert_shift+button_h+3*schrink;
% position 3 and 4 have been rescaled, we recompute to be on safe side
position_array	= [position(1)+schrink    position(2)+button_h+2*schrink...
		position(3)-2*schrink   position(4)-button_h-3*schrink];
dim = [(position_array(3)-horiz_shift)/nc    (position_array(4)-vert_shift)/nr];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% title at bottom + OK button
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

ik = 0;

% display the overall frame
ik = ik + 1;
posit = position;  
handle(ik) = uicontrol(...
	'style','frame',...
	'position',posit);		

% drop a push button at bottom 
posit(1) = position(1) + (position(3)-button_w)/2 ;	
posit(2) = position(2) + schrink;
posit(3) = button_w  ; 
posit(4) = button_h; % space for the button
ik = ik + 1;
handle(ik) = uicontrol(...
	'style','pushbutton',...
	'position',posit,...
	'string',title,...
	'CallBack','uiresume');

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% putting at top, column labels, at left row labels
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
cells_handles = []; % storing the handles of the cells
% dimension of each square   WHAT IF TOO SMALL????

posit = [position_array(1)+horiz_shift    position_array(2)+position_array(4)-vert_shift...
	dim(1)    vert_shift];
for j = 1:nc
	ik = ik + 1;
		 % using Edit implies frame!!!
	handle(ik) = uicontrol(...
		'style','Edit',...
		'position',posit ,...
		'string',label_col{j} );	
	posit(1) = posit(1) + dim(1);
end

position_array(4) = position_array(4) - vert_shift; 
posit = [ position_array(1) position_array(2)+position_array(4)-dim(2)  ...
	horiz_shift  dim(2) ];
for i = 1:nr
	ik = ik + 1;
		 % using Edit implies frame!!!
	handle(ik) = uicontrol(...
		'style','Edit',...
		'position',posit ,...
		'string',label_row{i} );	
	posit(2) = posit(2) - dim(2);
end	

position_array(1) = position_array(1) + horiz_shift;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% drawing the cells
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

posit = [position_array(1) position_array(2)+position_array(4) dim(1) dim(2)];
for i = 1:nr
	posit(2) = posit(2) - dim(2);
	posit(1) = position_array(1);
	for j = 1:nc
		ik = ik + 1;
		if option >= 2 ; textt = text(i,j);else; textt = '';end
		handle(ik) = uicontrol(...
			'style',style,...
			'position',posit ,...
			'string',textt );	
		posit(1) = posit(1) + dim(1);
		cells_handles =[ cells_handles ik];
	end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% I wait till OK pushed, meanwhile you can (re)fill the cells
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

uiwait(gcf)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% getting the results from the cells
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch option
case 1
	for i= 1:length(cells_handles)
		answer(i) = get(handle(cells_handles(i)), 'Value') ;
	end
case 2 
	for i= 1:length(cells_handles)
		answer(i) = str2num( get(handle(cells_handles(i)), 'String'));
		% nothing is done here if you don't feed a number XXXXXXX
	end
case 3
	for i= 1:length(cells_handles)
		answer{i} = get(handle(cells_handles(i)), 'String') ;
	end
end
answer = (reshape(answer,nc,nr))';	
delete(handle);
